@@ -8,3 +8,6 @@ from . import plugin
 from . import topology
 from . import main
 from . import config
+from . import components
+from . import tools
+from . import validate
