@@ -9,12 +9,13 @@ import re
 
 from box import Box
 
-from .. import common
+from ..utils import log
 from .. import data
 from .. import modules
 from ..modules import bgp
-from ..data.validate import validate_attributes
-from ..data.types import must_be_dict,must_be_list,must_be_string,must_be_id
+from ..data import get_box,get_empty_box
+from ..data.validate import validate_attributes,get_object_attributes
+from ..data.types import must_be_dict,must_be_list,must_be_id,transform_asdot
 from . import nodes
 
 '''
@@ -23,16 +24,17 @@ Return members of the specified group. Recurse through child groups if needed
 def group_members(topology: Box, group: str, count: int = 0) -> list:
   members: typing.List[str] = []
   if not group in topology.groups:  # pragma: no cover (just-in case catch, impossible to get here)
-    common.error(
+    log.error(
       f'Internal error: unknown group {group}',
-      common.IncorrectValue,
+      log.IncorrectValue,
       'groups')
     return []
 
   if count > 99:                    # pragma: no cover (impossible to get here, recursive groups are checked elsewhere)
-    common.fatal(
+    log.fatal(
       'Recursive group definition, aborting',
-      'groups')
+      module='groups',
+      header=True)
 
   for m in topology.groups[group].members:
     if m in topology.nodes:
@@ -44,25 +46,66 @@ def group_members(topology: Box, group: str, count: int = 0) -> list:
 
 '''
 Check validity of 'groups' data structure
-'''
-def check_group_data_structure(topology: Box) -> None:
-  if not 'groups' in topology:
-    topology.groups = Box({},default_box=True,box_dots=True)
 
-  if must_be_dict(topology,'groups','topology',create_empty=True,module='groups') is None:
-    return
+* groups -- the group data (topology- or default groups)
+* topology -- top-level topology (have to pass it to get attributes from fixed location)
+* prune_members -- remove non-existent group members (used for default groups)
+
+The checks are implemented in two functions:
+
+* check_group_data_sanity: called very early in the transformation process to do basic sanity
+  checks needed before we can auto-create group members (if required)
+* check_group-data_structure: called later when we already known topology-wide modules
+  and can perform full node attribute validation
+'''
+def check_group_data_sanity(
+      topology: Box,
+      parent_path: typing.Optional[str] = '') -> bool:
+
+  parent = topology.get(parent_path) if parent_path else topology
+  grp_namespace = f'{parent_path} ' if parent_path else ''
+
+  if must_be_dict(parent,'groups',parent_path,create_empty=True,module='groups') is None:
+    return False
 
   '''
   Transform group-as-list into group-as-dictionary
   '''
-  for grp in topology.groups.keys():
-    if isinstance(topology.groups[grp],list):
-      topology.groups[grp] = { 'members': topology.groups[grp] }
-    if grp in topology.nodes:
-      common.error(
-        f"group {grp} is also a node name. I can't deal with that level of confusion",
-        common.IncorrectValue,
+  for grp in parent.groups.keys():
+    if grp.startswith('_'):                       # Skip stuff starting with underscore
+      continue                                    # ... could be system settings
+
+    gpath = f'{parent_path or "topology"}.groups.{grp}'
+    if parent.groups[grp] is None:
+      log.error(
+        f"Definition of {grp_namespace}group '{grp}' must be a dictionary",
+        log.IncorrectType,
         'groups')
+      return False
+
+    if isinstance(parent.groups[grp],list):
+      parent.groups[grp] = { 'members': parent.groups[grp] }
+
+    if grp in topology.get('nodes',{}):
+      log.error(
+        f"{grp_namespace}group '{grp}' is also a node name. I can't deal with that level of confusion",
+        log.IncorrectValue,
+        'groups')
+      return False
+
+    if 'members' in parent.groups[grp]:
+      if must_be_list(parent.groups[grp],'members',gpath,create_empty=False,module='groups') is None:
+        return False
+
+  return True
+
+def check_group_data_structure(
+      topology: Box,
+      parent_path: typing.Optional[str] = '',
+      prune_members: typing.Optional[bool] = False) -> None:
+
+  parent = topology.get(parent_path) if parent_path else topology
+  grp_namespace = f'{parent_path} ' if parent_path else ''
 
   '''
   Sanity checks on global group data
@@ -70,15 +113,23 @@ def check_group_data_structure(topology: Box) -> None:
 
   list_of_modules = modules.list_of_modules(topology)
   group_attr = topology.defaults.attributes.group
-  providers = list(topology.defaults.providers.keys())
-  for grp,gdata in topology.groups.items():
+
+  # Allow provider- and tool- specific node attributes
+  extra = get_object_attributes(['providers','tools'],topology)
+
+  for grp,gdata in parent.groups.items():
+    if grp.startswith('_'):                       # Skip stuff starting with underscore
+      continue                                    # ... could be system settings
+
     must_be_id(parent=None,key=grp,path=f'NOATTR:group name {grp}',module='groups')
-    if must_be_dict(topology.groups,grp,'topology.groups',create_empty=True,module='groups') is None:
+
+    gpath = f'{parent_path or "topology"}.groups'
+    if must_be_dict(parent.groups,grp,gpath,create_empty=True,module='groups') is None:
       continue
 
-    gpath=f'topology.groups.{grp}'
+    gpath=f'{gpath}.{grp}'
     g_modules = gdata.get('module',[])
-    if g_modules:                          # Modules specified in the group -- we know what these nodes will use
+    if g_modules:                           # Modules specified in the group -- we know what these nodes will use
       gm_source = 'group'
     else:
       gm_source = 'topology'
@@ -93,21 +144,26 @@ def check_group_data_structure(topology: Box) -> None:
       module='groups',
       modules=g_modules,
       module_source=gm_source,
-      extra_attributes=providers)          # Allow provider-specific settings (not checked at the moment)
+      ignored=['_','netlab_','ansible_'],   # Ignore attributes starting with _, netlab_ or ansible_
+      extra_attributes=extra)               # Allow provider- and tool-specific settings (not checked at the moment)
 
     if not 'members' in gdata:
       gdata.members = []
 
     if grp == 'all' and gdata.members:
-      common.error('Group "all" should not have explicit members')
+      log.error(
+        text=f'{grp_namespace}group "all" should not have explicit members',
+        category=log.IncorrectValue,
+        module='groups')
 
-    must_be_dict(gdata,'vars',gpath,create_empty=False,module='groups')
-    must_be_dict(gdata,'node_data',gpath,create_empty=False,module='groups')
-    must_be_list(gdata,'config',gpath,create_empty=False,module='groups')
-    must_be_list(gdata,'module',gpath,create_empty=False,module='groups',valid_values=list_of_modules)
-    must_be_string(gdata,'device',gpath,module='groups',valid_values=list(topology.defaults.devices))
+    must_be_list(gdata,'module',gpath,create_empty=False,module='groups',valid_values=sorted(list_of_modules))
 
     if 'node_data' in gdata:                 # Validate node_data attributes (if any)
+      log.error(
+        text=f'Group {grp} uses an obsolete attribute node_data. Migrate node parameters into group definition',
+        category=Warning,
+        module='groups')
+
       validate_attributes(
         data=gdata.node_data,
         topology=topology,
@@ -117,15 +173,19 @@ def check_group_data_structure(topology: Box) -> None:
         module='groups',
         modules=g_modules,
         module_source=gm_source,
-        extra_attributes=providers)          # Allow provider-specific settings (not checked at the moment)
+        ignored=['_','netlab_','ansible_'],  # Ignore attributes starting with _, netlab_ or ansible_
+        extra_attributes=extra)              # Allow provider- and tool-specific settings (not checked at the moment)
 
       for k in ('module','device'):          # Check that the 'module' or 'device' attributes are not in node_data
         if k in gdata.node_data:
-          common.error(
-            f'Cannot use attribute {k} in node_data in group {grp}, set it as a group attribute',
-            common.IncorrectValue,
+          log.error(
+            f'Cannot use attribute {k} in node_data in {grp_namespace}group {grp}, set it as a group attribute',
+            log.IncorrectValue,
             'groups')
 
+    if log.pending_errors():                 # If we already found errors
+      continue                               # ... then the group data structures are not safe to work on
+
     for k in list(gdata.keys()):             # Then move (validated) group node attributes into node_data
       if k in group_attr:
         continue
@@ -135,9 +195,63 @@ def check_group_data_structure(topology: Box) -> None:
     if must_be_list(gdata,'members',gpath,create_empty=False,module='groups') is None:
       continue
 
-    for n in gdata.members:
-      if not n in topology.nodes and not n in topology.groups:
-        common.error('Member %s of group %s is not a valid node or group name' % (n,grp))
+    if prune_members:
+      gdata.members = [ n for n in gdata.members if n in topology.nodes or n in parent.groups ]
+    else:
+      for n in gdata.members:
+        if not n in topology.nodes and not n in parent.groups:
+          log.error(
+            text=f'Member {n} of {grp_namespace}group {grp} is not a valid node or group name',
+            category=log.IncorrectValue,
+            module='groups')
+
+'''
+Auto-create group members
+
+Inputs:
+* parent object (topology or defaults)
+* topology (needed to add nodes)
+* Global auto-create setting (set to 'False' for default groups)
+'''
+
+def auto_create_members(
+      parent: Box,
+      topology: Box,
+      default_create: bool) -> None:
+
+  if not 'groups' in parent:                      # Parent has no groups, what are we doing here?
+    return
+
+  parent_path = 'groups' if parent is topology else 'defaults.groups'
+  for gname,gdata in parent.groups.items():       # Iterate over groups
+    if gname.startswith('_'):                     # Skip stuff starting with '_' (could be global settings)
+      continue
+
+    if not 'members' in gdata:                    # Does the current group have members?
+      continue                                    # ... nope, nothing to create
+
+    # _auto_create could be set on the group or in global settings
+    #
+    if not gdata.get('_auto_create',False) and not default_create:
+      continue                                    # No auto-create for this group, move on
+
+    for n in gdata.members:                       # Iterate over group members
+      #
+      # Do not create nodes if the group member is another group
+      if n in topology.get('groups',{}) or n in topology.get('defaults.groups',{}):
+        continue
+
+      if n in topology.nodes:                     # Skip if the member is already a known node
+        continue
+
+      topology.nodes[n].name = n                  # Otherwise create an empty node data structure
+      topology.nodes[n].interfaces = []           # ... with an empty interface list
+  
+  '''
+  Transform group-as-list into group-as-dictionary
+  '''
+  for grp in parent.groups.keys():
+    gpath = f'{parent_path or "topology"}.groups.{grp}'
 
 '''
 Add node-level group settings to global groups
@@ -162,7 +276,7 @@ Check recursive group definitions
 
 def check_recursive_chain(topology: Box, chain: list, group: str) -> typing.Optional[list]:
   if not group in topology.groups: # pragma: no cover (if we ever get here we're seriously messed up)
-    common.fatal(
+    log.fatal(
       'Internal error: unknown group in check_recursive_chain')
     return None
 
@@ -170,7 +284,7 @@ def check_recursive_chain(topology: Box, chain: list, group: str) -> typing.Opti
   for m in topology.groups[group].members:
     if m in chain:
       chain = chain + [ m ]
-      common.error(f'Recursive group definition chain {chain}', common.IncorrectValue, 'groups')
+      log.error(f'Recursive group definition chain {chain}', log.IncorrectValue, 'groups')
       return chain
     if m in topology.groups:
       if check_recursive_chain(topology,chain,m):
@@ -180,12 +294,19 @@ def check_recursive_chain(topology: Box, chain: list, group: str) -> typing.Opti
 
 def check_recursive_groups(topology : Box) -> None:
   for gname in topology.groups.keys():
+    if gname.startswith('_'):                  # Skip settings starting with underscore
+      continue
     if check_recursive_chain(topology,[],gname):
       return
 
 def reverse_topsort(topology: Box) -> list:
-  group_copy = Box(topology.groups)            # Make a copy of the group dictionary
+  group_copy = get_box(topology.groups)        # Make a copy of the group dictionary
   sort_list: typing.List[str] = []
+
+  for g in list(group_copy.keys()):            # Clean up the group data structure
+    if g.startswith('_'):                      # Removing everything starting with underscore
+      group_copy.pop(g,None)                   # ... those are settings, not groups
+
   while group_copy:                            # Keep iterating until we got all groups in order
     for g in sorted(group_copy.keys()):        # Iterate over remaining groups
       OK_to_add = True                         # ... sort them by name to have consistent results
@@ -207,13 +328,13 @@ def copy_group_device_module(topology: Box) -> None:
     if not 'device' in gdata and not 'module' in gdata:
       continue                                                        # This group is not interesting, move on
 
-    if common.debug_active('groups'):
+    if log.debug_active('groups'):
       print(f'Setting device/module for group {grp}')
     g_members = group_members(topology,grp)
-    if not g_members:
-      common.error(
+    if not g_members and not gdata.get('_default_group',False):
+      log.error(
         f'Cannot use "module" or "device" attribute on in group {grp} that has no direct or indirect members',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'groups')
       continue
 
@@ -224,7 +345,7 @@ def copy_group_device_module(topology: Box) -> None:
       ndata = topology.nodes[name]
       if 'device' in gdata and not 'device' in ndata:                 # Copy device from group data to node data
         ndata.device = gdata.device
-        if common.debug_active('groups'):
+        if log.debug_active('groups'):
           print(f'... setting {name}.device to {gdata.device}')
 
       if 'module' in gdata:                                           # Merge group modules with device modules
@@ -233,13 +354,14 @@ def copy_group_device_module(topology: Box) -> None:
           if not m in ndata.module:                                   # ... and add missing modules to nodes
             ndata.module.append(m)
 
-        if common.debug_active('groups'):
+        if log.debug_active('groups'):
           print(f'... adding module {gdata.module} to {name}. Node modules now {ndata.module}')
 
 '''
 Copy node data from group into group members
 '''
 def copy_group_node_data(topology: Box,pfx: str) -> None:
+  topo_modules = topology.get('module',[])                            # Get list of default list of modules
   for grp in reverse_topsort(topology):
     if not grp.startswith(pfx):                                       # Skip groups that don't match the current prefix (ex: BGP autogroups)
       continue
@@ -248,15 +370,21 @@ def copy_group_node_data(topology: Box,pfx: str) -> None:
       continue
 
     g_members = group_members(topology,grp)                           # Get recursive list of members
-    if common.debug_active('groups'):
+    if log.debug_active('groups'):
       print(f'copy node data {grp}: {gdata.node_data}')
     for name in g_members:                                            # Iterate over group members
       if not name in topology.nodes:                                  # Member is not a node, skip it
         continue
 
-      if common.debug_active('groups'):
+      if log.debug_active('groups'):
         print(f'... merging node data with {name}')
-      topology.nodes[name] = gdata.node_data + topology.nodes[name]
+      merge_data = data.get_box(gdata.node_data)
+      if 'module' in topology.nodes[name]:
+        for m in topo_modules:
+          if not m in topology.nodes[name].module:
+            merge_data.pop(m,None)
+
+      topology.nodes[name] = merge_data + topology.nodes[name]
 
 '''
 Export node_data from groups to topology
@@ -280,23 +408,27 @@ def export_group_node_data(
   if not unique_keys:
     unique_keys = copy_keys
   for gname,gdata in topology.groups.items():
+    if gname.startswith('_'):                                         # Skip groups starting with '_'
+      continue                                                        # ... those are group settings
     #
     # Find groups with node_data dictionaries
+    # and check that the key within node_data dictionary we're interested in is also a dictionary
+    #
     if must_be_dict(gdata,f'node_data.{key}',f'groups.{gname}',module=module,create_empty=False):
-      for obj_name in list(gdata.node_data[key].keys()):
-        if gdata.node_data[key][obj_name] is None:
-          gdata.node_data[key][obj_name] = {}
-        obj_data =  gdata.node_data[key][obj_name]
+      for obj_name in list(gdata.node_data[key].keys()):              # Iterate over VLANs/VRFs within the group
+        if gdata.node_data[key][obj_name] is None:                    # ... replace None values with
+          gdata.node_data[key][obj_name] = {}                         # ... empty dictionaries
+        obj_data =  gdata.node_data[key][obj_name]                    # Now get the data to work with
         if not obj_name in topology[key] or topology[key][obj_name] is None:
-          topology[key][obj_name] = {}
-        for attr in unique_keys:
+          topology[key][obj_name] = {}                                # Make sure global object is also a dictionary
+        for attr in unique_keys:                                      # Check whether we have an overlap in unique keys
           if attr in topology[key][obj_name] and attr in obj_data and \
-             topology[key][obj_name][attr] != obj_data[attr]:                          # Unique key present on both ends and not equal
-            common.error(
+             topology[key][obj_name][attr] != obj_data[attr]:         # Unique key present on both ends and not equal
+            log.error(
               f'Cannot redefine {key} attribute {attr} for {key}.{obj_name} from node_data in group {gname}',
-              common.IncorrectValue,
+              log.IncorrectValue,
               module)
-        for attr in copy_keys:
+        for attr in copy_keys:                                        # Finally, copy missing values from group to global object
           if attr in obj_data and attr not in topology[key][obj_name]:
             topology[key][obj_name][attr] = obj_data[attr]
 
@@ -309,15 +441,13 @@ def create_bgp_autogroups(topology: Box) -> None:
   if not isinstance(g_module,list):                             # Won't deal with incorrectly formatted 'module' attribute here
     g_module = []                                               # ... just assume it's an empty list
   g_bgpas = data.get_global_parameter(topology,'bgp.as')        # Try to get the global BGP AS
-  if not data.is_true_int(g_bgpas):                             # ... don't worry if it's not int, someone else will complain
-    g_bgpas = 0
 
   for gname,gdata in topology.groups.items():                   # Sanity check: BGP autogroups should not have static members
     if re.match('as\\d+$',gname):
       if gdata.get('members',None):                             # Well, it's OK to have an empty list of members ;)
-        common.error(
+        log.error(
           f'BGP AS group {gname} should not have static members',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'groups')
 
   for n_name,n_data in topology.nodes.items():                  # Now iterate over nodes
@@ -328,7 +458,13 @@ def create_bgp_autogroups(topology: Box) -> None:
     if not 'bgp' in n_module:                                   # Looks like this node does not care about BGP
       continue
 
-    n_bgpas = data.get_from_box(n_data,'bgp.as') or g_bgpas     # Get node-level or global BGP AS
+    try:
+      n_bgpas = n_data.get('bgp.as',g_bgpas)                    # Get node-level or global BGP AS
+      if not data.is_true_int(n_bgpas):                         # ... if it's not int, it might be as.dot
+        n_bgpas = transform_asdot(n_bgpas)
+    except:
+      n_bgpas = 0
+
     if not n_bgpas:
       continue
 
@@ -338,30 +474,67 @@ def create_bgp_autogroups(topology: Box) -> None:
 
     topology.groups[grpname].members.append(n_name)             # ... and append the node to AS group
 
-#
-# init_groups:
-#
-# * Check and adjust group data structures
-# * Check recursive groups
-# * Add nodes to groups based on node 'group' attribute
-#
-# Please note that check_group_data_structure creates 'groups' element if needed
-# and 'adjust_groups' deletes it if there are no groups in the topology.
-#
+"""
+precheck_groups:
+
+* Check the baseline group data structure sanity
+* Auto-creates group members
+
+This function is called very early in the transformation process. Be very
+careful when touching other data structures and don't trust anything. Most
+of the topology data hasn't been validated yet.
+"""
+def precheck_groups(topology: Box) -> None:
+  auto_create_default = \
+    topology.get('defaults.groups._auto_create',False) or \
+    topology.get('groups._auto_create',False)     # auto-create could be set in defaults or on groups
+
+  if 'groups' in topology:
+    if check_group_data_sanity(topology):
+      auto_create_members(topology,topology,auto_create_default)
+
+  if 'groups' in topology.defaults:
+    if check_group_data_sanity(topology,'defaults'):
+      auto_create_members(topology.defaults,topology,False)
+
+"""
+init_groups:
+
+* Check and adjust group data structures
+* Check recursive groups
+* Add nodes to groups based on node 'group' attribute
+
+Please note that check_group_data_structure creates 'groups' element if needed
+and 'adjust_groups' deletes it if there are no groups in the topology.
+"""
+
 def init_groups(topology: Box) -> None:
-  check_group_data_structure(topology)
+  if 'groups' in topology:
+    check_group_data_structure(topology)
+
+  if 'groups' in topology.defaults:
+    check_group_data_structure(topology,'defaults',prune_members=True)
+    for gname,gdata in topology.defaults.groups.items():
+      if gname.find('_') == 0:
+        continue
+
+      if not gname in topology.groups:
+        gdata._default_group = True
+
+      topology.groups[gname] = gdata + topology.groups[gname]
+
   add_node_level_groups(topology)
-  common.exit_on_error()
+  log.exit_on_error()
 
   check_recursive_groups(topology)
-  common.exit_on_error()
+  log.exit_on_error()
 
   copy_group_device_module(topology)
   copy_group_node_data(topology,'')                 # Copy all group data into nodes (potentially setting bgp.as)
   bgp.process_as_list(topology)
   create_bgp_autogroups(topology)                   # Create AS-based groups
   copy_group_node_data(topology,'as')               # And add group data from 'asxxxx' into nodes
-  common.exit_on_error()
+  log.exit_on_error()
   if not topology.groups:
     del topology['groups']
 
@@ -401,6 +574,9 @@ def node_config_templates(topology: Box) -> None:
   '''
 
   for group_name in topology.groups:
+    if group_name.startswith('_'):                # Skip settings
+      continue
+
     topology.groups[group_name].pop('config',None)
 
   for name,node in topology.nodes.items():
@@ -420,3 +596,19 @@ def node_config_templates(topology: Box) -> None:
       node.config = config_list
     else:
       node.pop('config',None)
+
+"""
+Final step in group processing: remove settings from groups data structure,
+so the templates don't have to guess whether they're dealing with groups
+or settings
+"""
+def cleanup(topology: Box) -> None:
+  if not 'groups' in topology:                    # No groups, no worries
+    return
+
+  for gname in list(topology.groups.keys()):      # Iterate over group names
+    if gname.startswith('_'):                     # ... and if something starts with underscore, it's a setting
+      topology.groups.pop(gname,None)             # ... so remove it
+
+  if not topology.groups:                         # Anything left?
+    topology.pop('groups',None)                   # ... nope, remove the whole 'groups' thingy
