@@ -8,13 +8,14 @@ import netaddr
 from box import Box
 
 # Related modules
-from .. import common
+from ..utils import log,strings
 from .. import data
-from .. import utils
-from ..data.validate import validate_attributes
+from ..data.validate import validate_attributes,get_object_attributes
 from ..data.types import must_be_string,must_be_list,must_be_dict,must_be_id
-from .. import addressing
-from . import devices
+from . import devices,addressing
+
+VIRTUAL_INTERFACE_TYPES: typing.Final[typing.List[str]] = [
+  'loopback', 'tunnel' ]
 
 def adjust_interface_list(iflist: list, link: Box, nodes: Box) -> list:
   link_intf = []
@@ -23,28 +24,28 @@ def adjust_interface_list(iflist: list, link: Box, nodes: Box) -> list:
     intf_cnt = intf_cnt + 1
     if isinstance(n,str):               # Another shortcut: node name as string
       if not n in nodes:                # ... is it a valid node name?
-        common.error(                   # ... it's not, get lost
+        log.error(                   # ... it's not, get lost
           f'Interface {link._linkname}.interfaces.{intf_cnt} refers to an unknown node {n}',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'links')
         continue
-      n = Box({ 'node': n },default_box=True,box_dots=True)
+      n = data.get_box({ 'node': n })
       
     if not isinstance(n,Box):          # Still facing non-dict data type?
-      common.error(                     # ... report an error
+      log.error(                     # ... report an error
         f'Interface data in {link._linkname}.interfaces[{intf_cnt}] must be a dictionary, found {type(n).__name__}',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'links')
     elif not 'node' in n:               # Do we have node name in interface data?
-      common.error(                     # ... no? Too bad, throw an error
-        f'Interface data {link._linkname}.interfaces[{intf_cnt}] is missing a "node" attribute\n' +
-        f'... found {n}',
-        common.MissingValue,
-        'links')
+      log.error(                     # ... no? Too bad, throw an error
+        text=f'Interface data {link._linkname}.interfaces[{intf_cnt}] is missing a "node" attribute',
+        more_data=[ f'found {n}' ],
+        category=log.MissingValue,
+        module='links')
     elif not n.node in nodes:           # Is the node name valid?
-      common.error(                     # ... it's not, get lost
+      log.error(                     # ... it's not, get lost
         f'Interface data {link._linkname}.interfaces[{intf_cnt}] refers to an unknown node {n.node}',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'links')
     else:
       link_intf.append(n)               # Interface data is OK, append it to interface list
@@ -94,24 +95,29 @@ def adjust_link_object(l: typing.Any, linkname: str, nodes: Box) -> typing.Optio
   if isinstance(l,str):                                       # String, split into a list of nodes
     link_intf = []
     for n in l.split('-'):                # ... split it into a list of nodes
-      if n in nodes:                      # If the node name is valid
+      n = n.strip()                       # ... strip leading and trailing spaces (fixing #816)
+      valid_node = n in nodes
+      if not valid_node:
+        valid_node = len([ x for x in nodes if n.startswith(x) ]) > 0
+
+      if valid_node:                      # If the node name is valid
         link_intf.append({ 'node': n })   # ... append it to the list of interfaces
       else:
-        common.error(
+        log.error(
           f'Link string {l} in {linkname} refers to an unknown node {n}',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'links')
     return data.get_box({
       'interfaces': link_intf,
       '_linkname' : linkname })
 
-  common.error(
+  log.error(
     f'Invalid type {type(l).__name__} for {linkname}',
-    common.IncorrectType,
+    log.IncorrectType,
     'links')
   return None
 
-def adjust_link_list(links: list, nodes: Box) -> list:
+def adjust_link_list(links: list, nodes: Box, linkname_format: str = 'links[{link_cnt}]') -> list:
   link_list: list = []
 
   if not(links):
@@ -120,14 +126,17 @@ def adjust_link_list(links: list, nodes: Box) -> list:
   link_cnt = 0
   for l in links:
     link_cnt = link_cnt + 1
-    link_data = adjust_link_object(l,f'links[{link_cnt}]',nodes)
+    linkname = linkname_format.format(link_cnt=link_cnt)
+    link_data = adjust_link_object(l,linkname,nodes)
     if not link_data is None:
+      if link_data.get('disable',False) is True:
+        continue
       link_list.append(link_data)
 
-  if common.debug_active('links'):
+  if log.debug_active('links'):
     print("Adjusted link list")
     print("=" * 60)
-    print(common.get_yaml_string(link_list))
+    print(strings.get_yaml_string(link_list))
 
   return link_list
 
@@ -135,7 +144,9 @@ def adjust_link_list(links: list, nodes: Box) -> list:
 Validate link attributes
 """
 def validate(topology: Box) -> None:
-  providers = list(topology.defaults.providers.keys())
+  # Allow provider-specific global attributes
+  providers = get_object_attributes(['providers'],topology)
+
   for l_data in topology.links:
     validate_attributes(
       data=l_data,                                    # Validate link data
@@ -157,6 +168,7 @@ def validate(topology: Box) -> None:
         data_name=f'interface',
         attr_list=['interface','link'],                 # We're checking interface or link attributes
         modules=n_data.get('module',[]),                # ... against node modules
+        extra_attributes=providers,                     # Allow provider-specific attributes in interfaces
         module_source=f'nodes.{intf.node}',
         module='links')                                 # Function is called from 'links' module
 
@@ -168,7 +180,7 @@ def validate(topology: Box) -> None:
         data_path=f'{l_data._linkname}.prefix',         # Topology path to link prefix
         data_name=f'prefix',
         attr_list=['prefix'],                           # We're checking prefix attributes
-        modules=None,                                   # No module attributes in prefix
+        modules=[],                                     # No module attributes in prefix
         module='links')
 
 """
@@ -198,48 +210,64 @@ def create_regular_interface(node: Box, ifdata: Box, defaults: Box) -> None:
 
   # When computing default ifindex, consider only non-loopback interfaces
   if not ifindex:
-   ifindex = len([intf for intf in node.interfaces if intf.get('type',None) != 'loopback']) + ifindex_offset
+   ifindex = len([
+               intf for intf in node.interfaces
+                 if not intf.get('type',None) in VIRTUAL_INTERFACE_TYPES
+                 ]) + ifindex_offset
 
   ifname_format = devices.get_device_attribute(node,'interface_name',defaults)
 
   ifdata.ifindex = ifindex
   if ifname_format and not 'ifname' in ifdata:
-    ifdata.ifname = utils.strings.eval_format(ifname_format,ifdata)
+    ifdata.ifname = strings.eval_format(ifname_format,ifdata)
 
   pdata = devices.get_provider_data(node,defaults).get('interface',{})
-  pdata = Box(pdata,box_dots=True,default_box=True)                     # Create a copy of the provider interface data
+  pdata = data.get_box(pdata)                     # Create a copy of the provider interface data
   if 'name' in pdata:
-    pdata.name = utils.strings.eval_format(pdata.name,ifdata)
+    pdata.name = strings.eval_format(pdata.name,ifdata)
 
   if pdata:
     provider = devices.get_provider(node,defaults)
     ifdata[provider] = pdata
 
-def create_loopback_interface(node: Box, ifdata: Box, defaults: Box) -> None:
-  ifindex_offset = devices.get_device_attribute(node,'loopback_offset',defaults) or 0
-  ifdata.ifindex = ifindex_offset + ifdata.linkindex
-  ifdata.virtual_interface = True
-  ifname_format  = devices.get_device_attribute(node,'loopback_interface_name',defaults)
+def create_virtual_interface(node: Box, ifdata: Box, defaults: Box) -> None:
+  devtype = ifdata.get('type','loopback')         # Get virtual interface type, default to loopback interface
+  ifindex_offset = (
+    devices.get_device_attribute(node,f'{devtype}_offset',defaults) or
+    1 if devtype == 'loopback' else 0)            # Loopback interfaces have to start with 1 to prevent overlap with built-in loopback
 
-  if not ifname_format:
-    common.error(
-      f'Device {node.device}/node {node.name} does not support loopback links',
-      common.IncorrectValue,
-      'links')
-    return
+  ifdata.virtual_interface = True
+  ifdata.pop('bridge',None)
+
+  if not 'ifindex' in ifdata:
+    ifdata.ifindex = len([intf for intf in node.interfaces if intf.get('type',None) == devtype]) + ifindex_offset
+  ifname_format  = devices.get_device_attribute(node,f'{devtype}_interface_name',defaults)
 
   if not 'ifname' in ifdata:
-    ifdata.ifname = utils.strings.eval_format(ifname_format,ifdata)
+    if not ifname_format:
+      if devtype == 'loopback':
+        log.error(
+          f'Device {node.device}/node {node.name} does not support loopback links',
+          log.IncorrectValue,
+          'links')
+        return
+      else:
+        print(ifdata)
+        log.error(
+          f'Need explicit interface name (ifname) for {devtype} interface on node {node.name} ({node.device})',
+          log.IncorrectValue,
+          'links')
+        return
+    ifdata.ifname = strings.eval_format(ifname_format,ifdata)
 
-  # If the device uses 'loopback_offset' then we're assuming it's large enough to prevent overlap with physical interfaces
-  # Otherwise, create fake ifindex for loopback interfaces to prevent that overlap
+  # Adjust ifindex to prevent overlap between device types
   #
-  if not ifindex_offset:
-    ifdata.ifindex = 10000 + ifdata.ifindex
+  ifindex_offset = (VIRTUAL_INTERFACE_TYPES.index(devtype)+1) * 10000
+  ifdata.ifindex += ifindex_offset
 
 def add_node_interface(node: Box, ifdata: Box, defaults: Box) -> Box:
-  if ifdata.get('type') == 'loopback':
-    create_loopback_interface(node,ifdata,defaults)
+  if ifdata.get('type',None) in VIRTUAL_INTERFACE_TYPES:
+    create_virtual_interface(node,ifdata,defaults)
   else:
     create_regular_interface(node,ifdata,defaults)
 
@@ -276,17 +304,25 @@ def interface_data(link: Box, link_attr: set, ifdata: Box) -> Box:
         ifdata[k] = link[k] + ifdata[k]
   return ifdata
 
+"""
+Get gateway ID -- return None if not set or if 'gateway' is not a dict
+"""
+def get_gateway_id(link: Box) -> typing.Optional[int]:
+  if not isinstance(link.get('gateway',None),Box):
+    return None
+  return link.gateway.get('id',None)
+
 """
 Set FHRP (anycast/VRRP/...) gateway on a link
 """
 
 def set_fhrp_gateway(link: Box, pfx_list: Box, nodes: Box, link_path: str) -> None:
-  gwid = data.get_from_box(link,'gateway.id')
+  gwid = get_gateway_id(link)
   if not gwid:                                                        # No usable gateway ID, nothing to do
     return
 
   fhrp_assigned = False
-  for af in common.AF_LIST:
+  for af in log.AF_LIST:
     if not af in pfx_list or isinstance(pfx_list[af],bool):           # No usable IPv4/IPv6 prefix, nothing to do
       continue
 
@@ -294,11 +330,11 @@ def set_fhrp_gateway(link: Box, pfx_list: Box, nodes: Box, link_path: str) -> No
       link.gateway[af] = get_nth_ip_from_prefix(netaddr.IPNetwork(link.prefix[af]),link.gateway.id)
       fhrp_assigned = True
     except Exception as ex:
-      common.error(
+      log.error(
         f'Cannot generate gateway IP address on {link_path}' + \
         f' from [af] prefix {link.prefix[af]} and gateway ID {link.gateway.id}\n' + \
-        common.extra_data_printout(str(ex)),
-        common.IncorrectValue,
+        strings.extra_data_printout(str(ex)),
+        log.IncorrectValue,
         'links')
       return
 
@@ -307,11 +343,11 @@ def set_fhrp_gateway(link: Box, pfx_list: Box, nodes: Box, link_path: str) -> No
 
   for intf in link.interfaces:                                        # Copy link gateway into interface attributes
     if 'gateway' in nodes[intf.node].get('module',[]):                # ... but only for nodes using the gateway module
-      for af in common.AF_LIST:
+      for af in log.AF_LIST:
         if af in link.gateway:
           intf.gateway[af] = link.gateway[af]
 
-  if common.debug_active('links'):     # pragma: no cover (debugging)
+  if log.debug_active('links'):     # pragma: no cover (debugging)
     print(f'FHRP gateway set for {link}')
 
 """
@@ -343,14 +379,14 @@ def assign_link_prefix(
     return pfx_list
 
   if 'unnumbered' in link:                                # User requested an unnumbered link
-    link.prefix = Box({ 'unnumbered': True })
+    link.prefix = data.get_box({ 'unnumbered': True })
     return link.prefix
 
   if must_be_string(link,'pool',link_path):
     if not link.pool in addr_pools:
-      common.error(
+      log.error(
         f'Unknown address pool {link.pool} used in {link_path}',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'links')
     pools = [ link.pool ] + pools
   else:
@@ -359,7 +395,7 @@ def assign_link_prefix(
 
   pfx_list = addressing.get(addr_pools,pools)
   link.prefix = {
-      af: str(v) if af in common.AF_LIST and not isinstance(v,bool) else v for af,v in pfx_list.items()
+      af: str(v) if af in log.AF_LIST and not isinstance(v,bool) else v for af,v in pfx_list.items()
     }
   if not link.prefix:
     link.pop('prefix',None)
@@ -381,7 +417,7 @@ def get_prefix_IPAM_policy(link: Box, pfx: typing.Union[netaddr.IPNetwork,bool],
   if isinstance(pfx,bool):
     return 'unnumbered'
 
-  gwid = data.get_from_box(link,'gateway.id') or 0                    # Get link gateway ID (if set) --- must be int for min to work
+  gwid = get_gateway_id(link) or 0                                    # Get link gateway ID (if set) --- must be int for min to work
   if link.type == 'p2p' and not gwid:                                 # P2P allocation policy cannot be used with default gateway
     return 'p2p' if pfx.first != pfx.last else 'error'
 
@@ -393,18 +429,18 @@ def get_prefix_IPAM_policy(link: Box, pfx: typing.Union[netaddr.IPNetwork,bool],
     if gwid > 0:                                                      # Gateway ID at the front of the subnet -- need one extra IP
       add_extra_ip = 1
     if gwid < 0:                                                      # Don't allow node address allocation beyond last-in-subnet gateway
-      subtract_reserved_ip = min(subtract_reserved_ip,gwid)
+      subtract_reserved_ip = min(subtract_reserved_ip,gwid-1)
 
     pfx_size = pfx_size + subtract_reserved_ip
 
   max_id = max([ ndict[intf.node].id for intf in link.interfaces if intf.node in ndict ])
-  if max_id < pfx_size:                                               # If we can fit all node IDs attached to this link into the prefix
-    return 'id_based'                                                 # ... we'll use ID-based address allocation
-  if len(link.interfaces) + add_extra_ip < pfx_size:                  # Otherwise, if the prefix is big enough
-    return 'sequential'                                               # ... we'll use sequential address allocation
+  if pfx_size == 1:                                                   # Do we have a single node attached to a /32 link?
+    return 'loopback' if len(link.interfaces) == 1 else 'error'       # ... if so, we'll use loopback address allocation
 
-  if pfx_size < 2 and link.type == 'loopback':                        # Final straw: maybe we're dealing with a loopback?
-    return 'loopback'
+  if max_id <= pfx_size:                                              # If we can fit all node IDs attached to this link into the prefix
+    return 'id_based'                                                 # ... we'll use ID-based address allocation
+  if len(link.interfaces) + add_extra_ip <= pfx_size:                 # Otherwise, if the prefix is big enough
+    return 'sequential'                                               # ... we'll use sequential address allocation
 
   return 'error'
 
@@ -438,9 +474,9 @@ def set_interface_address(intf: Box, af: str, pfx: netaddr.IPNetwork, node_id: i
           intf_pfx.prefixlen = pfx.prefixlen    # If it lacks a prefix, add link prefix
         intf[af] = str(intf_pfx)                # ... and save modified/validated interface IP address
       except Exception as ex:
-        common.error(
-          f'Cannot parse {af} address {intf.af} for node {intf.node}\n'+common.extra_data_printout(str(ex)),
-          common.IncorrectValue,
+        log.error(
+          f'Cannot parse {af} address {intf.af} for node {intf.node}\n'+strings.extra_data_printout(str(ex)),
+          log.IncorrectValue,
           'links')
         return False
 
@@ -450,9 +486,9 @@ def set_interface_address(intf: Box, af: str, pfx: netaddr.IPNetwork, node_id: i
       if intf_pfx.last <= intf_pfx.first + 1:   # Are we dealing with special prefix (loopback or /31)
         return True                             # ... then it's OK not to have host bits
 
-      common.error(
-        f'Address {intf.af} for node {intf.node} does not contain host bits',
-        common.IncorrectValue,
+      log.error(
+        f'Address {intf[af]} for node {intf.node} does not contain host bits',
+        log.IncorrectValue,
         'links')
       return False
 
@@ -461,10 +497,10 @@ def set_interface_address(intf: Box, af: str, pfx: netaddr.IPNetwork, node_id: i
     intf[af] = get_nth_ip_from_prefix(pfx,node_id)
     return True
   except Exception as ex:
-    common.error(
+    log.error(
       f'Cannot assign host index {node_id} in {af} from prefix {str(pfx)} to node {intf.node}\n' + \
-          common.extra_data_printout(str(ex)),
-      common.IncorrectValue,
+          strings.extra_data_printout(str(ex)),
+      log.IncorrectValue,
       'links')
 
   return False
@@ -477,16 +513,16 @@ If the interface address is set, validate that it's a valid address (can't be in
 def IPAM_unnumbered(link: Box, af: str, pfx: typing.Optional[bool], ndict: Box) -> None:
   for intf in link.interfaces:
     if not af in intf:            # No static address, set it to link bool value or use loopback AF presence for old-style unnumbereds
-      intf[af] = pfx if isinstance(pfx,bool) else bool(data.get_from_box(ndict[intf.node],f'loopback.{af}'))
+      intf[af] = pfx if isinstance(pfx,bool) else bool(ndict[intf.node].get(f'loopback.{af}',False))
     elif data.is_true_int(intf[af]):
-      common.error(
+      log.error(
         f'Node {intf.node} is using host index {intf[af]} for {af} on an unnumbered link',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'links')
 
 def IPAM_sequential(link: Box, af: str, pfx: netaddr.IPNetwork, ndict: Box) -> None:
   start = 1 if pfx.last != pfx.first + 1 else 0
-  gwid = data.get_from_box(link,'gateway.id')
+  gwid = get_gateway_id(link)
   for count,intf in enumerate(link.interfaces):
     if count + start == gwid:                                   # Would the next address overlap with gateway ID
       start = start + 1                                         # ... no big deal, just move the starting point ;)
@@ -547,11 +583,11 @@ def assign_interface_addresses(link: Box, addr_pools: Box, ndict: Box, defaults:
       try:                                            # Parse the AF prefix
         pfx_net = netaddr.IPNetwork(pfx_list[af])
       except Exception as ex:                         # Report an error and move on if it cannot be parsed
-        common.error(
+        log.error(
           f'Cannot parse {af} prefix {pfx_list[af]} on {link._linkname}\n' + \
-            common.extra_data_printout(f'{ex}') + '\n' + \
-            common.extra_data_printout(f'{link}'),
-          common.IncorrectValue,
+            strings.extra_data_printout(f'{ex}') + '\n' + \
+            strings.extra_data_printout(f'{link}'),
+          log.IncorrectValue,
           'links')
         continue
 
@@ -562,19 +598,19 @@ def assign_interface_addresses(link: Box, addr_pools: Box, ndict: Box, defaults:
 
     if allocation_policy == 'error':                                      # Something went wrong, cannot assing IP addresses
       rq = f'{len(link.interfaces)} nodes'
-      if data.get_from_box(link,'gateway.id'):
+      if get_gateway_id(link):
         rq = rq + f' plus first-hop gateway'
-      common.error(
-        f'Cannot use {af} prefix {pfx_list[af]} to address {rq} on {link._linkname}\n' + \
-        common.extra_data_printout(f'link data: {link}',width=90),
-        common.IncorrectValue,
-        'links')
+      log.error(
+        f'Cannot use {af} prefix {pfx_list[af]} to address {rq} on {link._linkname}',
+        more_data=f'link data: {link}',
+        category=log.IncorrectValue,
+        module='links')
       continue
 
     if not allocation_policy in IPAM_dispatch:
-      common.error(
+      log.error(
         f'Invalid IP address allocation policy specified in prefix {pfx_list} found on {link._linkname}',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'links')
     IPAM_dispatch[allocation_policy](link,af,pfx_net,ndict)               # execute IPAM policy to get AF addresses on interfaces
 
@@ -595,18 +631,18 @@ def cleanup_link_interface_AF_entries(link: Box) -> None:
         continue
 
       if data.is_true_int(intf[af]):            # Unprocessed int. Must be node index on an unnumbered link ==> error
-        common.error(
+        log.error(
           f'Interface ID for node {intf.node} did not result in a usable address on {link._linkname}\n' + \
-            common.extra_data_printout(f'{link}'),
-          common.IncorrectType,
+            strings.extra_data_printout(f'{link}'),
+          log.IncorrectType,
           'links')
         continue
 
       if not '/' in intf[af]:                   # Subnet mask is unknown
-        common.error(
+        log.error(
           f'Unknown subnet mask for {af} address {intf[af]} used by node {intf.node} on {link._linkname}\n' + \
-            common.extra_data_printout(f'{link}'),
-          common.IncorrectType,
+            strings.extra_data_printout(f'{link}'),
+          log.IncorrectType,
           'links')
         continue
 
@@ -634,13 +670,28 @@ def set_interface_name(ifdata: Box, link: Box, ifcnt: int) -> None:
   else:
     ifdata.name = f'{node_name} -> [{",".join(list(n_list))}]'
 
+"""
+set_parent_interface -- set the parent interface and IP address for the IPv4 unnumbered interfaces
+"""
+def set_parent_interface(ifdata: Box, node: Box) -> None:
+  if ifdata.get('ipv4',None) is not True:                         # We're interested only in interfaces that have...
+    return                                                        # ... ipv4 set to True
+
+  lbname = node.get('loopback.ifname',None)                       # Try to get the parent interface name
+  if lbname:                                                      # Found it, save it in an internal attribute
+    ifdata._parent_intf = lbname
+
+  lbipv4 = node.get('loopback.ipv4',None)                         # Try to get the parent IPv4 address
+  if lbipv4:                                                      # Save the parent IPv4 address (needed for static routes)
+    ifdata._parent_ipv4 = lbipv4
+
 """
 Create node interfaces from link interfaces
 """
 def create_node_interfaces(link: Box, addr_pools: Box, ndict: Box, defaults: Box) -> None:
   link_attr_propagate = get_link_propagate_attributes(defaults)
 
-  if common.debug_active('links'):     # pragma: no cover (debugging)
+  if log.debug_active('links'):     # pragma: no cover (debugging)
     print(f'\nCreate node interfaces: {link} using {link_attr_propagate}')
 
   interfaces = []
@@ -652,8 +703,9 @@ def create_node_interfaces(link: Box, addr_pools: Box, ndict: Box, defaults: Box
     ifdata = interface_data(
                 link=link,
                 link_attr=link_attr_propagate.union(ndict[node].get('module',[])) - set(defaults.attributes.link_module_no_propagate),
-                ifdata=Box(value))
+                ifdata=data.get_box(value))
     set_interface_name(ifdata,link,intf_cnt)
+    set_parent_interface(ifdata,ndict[node])
     ifdata.pop('node',None)                                       # Remove the node name (not needed within the node)
     node_intf = add_node_interface(ndict[node],ifdata,defaults)   # Attach new interface to its node
     value.ifindex = node_intf.ifindex                             # Save ifindex and ifname in link interface data
@@ -661,7 +713,7 @@ def create_node_interfaces(link: Box, addr_pools: Box, ndict: Box, defaults: Box
     interfaces.append({ 'node': node, 'data': node_intf })        # Save newly-created interface for the next step
                                                                   # ... must use dict not Box as Box creates a copy of the data structure
 
-  if common.debug_active('links'):     # pragma: no cover (debugging)
+  if log.debug_active('links'):     # pragma: no cover (debugging)
     print(f'... link data: {link}')
     print(f'... interface data: {interfaces}\n')
 
@@ -674,7 +726,7 @@ def create_node_interfaces(link: Box, addr_pools: Box, ndict: Box, defaults: Box
         continue
       remote_node = remote_if['node']                             # Remote node name in a handier format
       remote_ifdata = remote_if['data']                           # ... and a pointer to remote interface data
-      ngh_data = Box({ 'ifname': remote_ifdata.ifname, 'node': remote_node })
+      ngh_data = data.get_box({ 'ifname': remote_ifdata.ifname, 'node': remote_node })
       #
       # Find relevant modules that have interface attributes
       mods_with_attr = set([ m for m in ndict[remote_node].get('module',[])
@@ -719,6 +771,9 @@ def set_link_type_role(link: Box, pools: Box, nodes: Box, defaults: Box) -> None
     if not 'role' in link and host_count == node_cnt - 1 and not 'vlan_name' in link:
       link.role = 'stub'
 
+  if link.get('dhcp.subnet.ipv4',None):
+    link.host_count = link.get('host_count',0) + 1
+
   if 'type' in link:                # Link type already set, nothing to do
     return
 
@@ -726,7 +781,7 @@ def set_link_type_role(link: Box, pools: Box, nodes: Box, defaults: Box) -> None
   if node_cnt == 1:
     set_link_loopback_type(link,nodes,defaults)
 
-  if host_count > 0:
+  if link.get('host_count',0) > 0:
     link.type = 'lan'
 
   return
@@ -737,9 +792,9 @@ def set_link_bridge_name(link: Box, defaults: Box) -> None:
   if not 'bridge' in link:
     link['bridge'] = "%s_%d" % (defaults.name[0:10],link.linkindex)   # max 15 chars on Linux
   elif len(link['bridge']) > 15:
-    common.error(
+    log.error(
       f'Bridge name {link["bridge"]} has more than 15 characters',
-      common.IncorrectValue,
+      log.IncorrectValue,
       'interfaces')
 
 def check_link_type(data: Box) -> bool:
@@ -747,37 +802,31 @@ def check_link_type(data: Box) -> bool:
   link_type = data.get('type')
 
   if 'mtu' in data and not isinstance(data.mtu,int): # pragma: no cover
-    common.error(f'MTU parameter should be an integer: {data}',common.IncorrectValue,'links')
+    log.error(f'MTU parameter should be an integer: {data}',log.IncorrectValue,'links')
 
   if not link_type: # pragma: no cover (shouldn't get here)
-    common.fatal('Internal error: link type still undefined in check_link_type: %s' % data,'links')
+    log.fatal('Internal error: link type still undefined in check_link_type: %s' % data,'links')
     return False
 
   if node_cnt == 0:
-    common.error('No valid nodes on link %s' % data,common.MissingValue,'links')
+    log.error('No valid nodes on link %s' % data,log.MissingValue,'links')
     return False
 
   if link_type == 'stub' and node_cnt > 1:
-    common.error('More than one node connected to a stub link: %s' % data,common.IncorrectValue,'links')
+    log.error('More than one node connected to a stub link: %s' % data,log.IncorrectValue,'links')
     return False
 
   if link_type == 'p2p' and node_cnt != 2:
-    common.error('Point-to-point link needs exactly two nodes: %s' % data,common.IncorrectValue,'links')
+    log.error('Point-to-point link needs exactly two nodes: %s' % data,log.IncorrectValue,'links')
     return False
 
   if link_type == 'loopback' and node_cnt != 1:
-    common.error(
+    log.error(
       f'Looopback link {data._linkname} can have a single node attached\n... {data}',
-      common.IncorrectValue,
+      log.IncorrectValue,
       'links')
     return False
 
-  if not link_type in [ 'stub','p2p','lan','loopback','vlan_member']:
-    common.error(
-      f'Invalid link type {link_type} in {data._linkname}\n... {data}',
-      common.IncorrectValue,
-      'links')
-    return False
   return True
 
 #
@@ -791,18 +840,18 @@ def interface_feature_check(nodes: Box, defaults: Box) -> None:
       if 'ipv4' in ifdata:
         if isinstance(ifdata.ipv4,bool) and ifdata.ipv4 and \
             not features.initial.ipv4.unnumbered:
-          common.error(
-            f'Device {ndata.device} does not support unnumbered IPv4 interfaces used on\n'+
-            f'.. node {node} interface {ifdata.ifname} (link {ifdata.name})',
-            common.IncorrectValue,
-            'interfaces')
+          log.error(
+            text=f'Device {ndata.device} does not support unnumbered IPv4 interfaces',
+            more_hints=[ f'Used on node {node} interface {ifdata.ifname} (link {ifdata.name})' ],
+            category=log.IncorrectValue,
+            module='interfaces')
       if 'ipv6' in ifdata:
         if isinstance(ifdata.ipv6,bool) and ifdata.ipv6 and \
             not features.initial.ipv6.lla:
-          common.error(
+          log.error(
             f'Device {ndata.device} does not support LLA-only IPv6 interfaces used on\n'+
             f'.. node {node} interface {ifdata.ifname} (link {ifdata.name})',
-            common.IncorrectValue,
+            log.IncorrectValue,
             'interfaces')
 
 def set_default_gateway(link: Box, nodes: Box) -> None:
@@ -814,26 +863,33 @@ def set_default_gateway(link: Box, nodes: Box) -> None:
   if not 'ipv4' in link.prefix or isinstance(link.prefix.ipv4,bool):
     return
 
-#  if 'vlan_name' in link:                                 # Do not try to set first-hop gateways on VLAN links, VLAN module will do that
-#    return
-
-  if common.debug_active('links'):
+  if log.debug_active('links'):
     print(f'Set DGW for {link}')
-  if not 'gateway' in link:
-    gateway = None
-    for ifdata in link.interfaces:
+  if not 'gateway' in link:                               # Do we have first-hop gateway on the link?
+    for ifdata in link.interfaces:                        # Nope, iterate over interfaces, find routers running IPv4
       if nodes[ifdata.node].get('role','') != 'host' and ifdata.get('ipv4',False):
-        link.gateway.ipv4 = ifdata.ipv4
-        break
+        if ifdata.get('dhcp.client.ipv4',False):
+          continue                                        # Skip routers running DHCP clients
+        link.gateway.ipv4 = ifdata.ipv4                   # Remember the router's IPv4 address
+        if ifdata.ipv4 is not True:                       # ... and if it's not unnumbered
+          break                                           # ... get out, we found it
+
+    if link.get('gateway.ipv4',None) is True:             # Did we find an unnumbered IPv4 address?
+      log.error(                                          # Complain...
+        text=f'Hosts cannot be attached to subnets where all routers have unnumbered interfaces (found in {link._linkname})',
+        category=log.IncorrectValue,
+        module='links')
+      link.pop('gateway',None)                            # ... and remove it, it's useless
+
   elif link.gateway is False:
     return
 
   if not 'gateway' in link or not isinstance(link.gateway,Box) or not 'ipv4' in link.gateway: # Didn't find a usable gateway, exit
-    if common.debug_active('links'):
+    if log.debug_active('links'):
       print('... not found')
     return
 
-  if common.debug_active('links'):
+  if log.debug_active('links'):
     print(f'... DGW: {link.gateway}')
 
   for ifdata in link.interfaces:                          # Copy link gateway to all hosts attached to the link
@@ -866,7 +922,7 @@ Link index utility functions:
 '''
 
 def get_next_linkindex(topology: Box) -> int:
-  if not 'links' in topology:
+  if not topology.links:
     topology.links = []
     return topology.defaults.get('link_index',1)
 
@@ -900,14 +956,17 @@ def set_linkindex(topology: Box) -> None:
 '''
 expand_groups -- expand link groups (identified by 'group' and 'members' attributes) into individual links
 appended to the end of the link list
+
+This function is called from init_links very early in the topology initialization process
+and must do its own data validation.
 '''
 def expand_groups(topology: Box) -> None:
   for link in list(topology.links):                 # Iterate over existing links (that's why we have to cast it as a list)
     if not 'group' in link:                         # Not a group link, move on
       if 'members' in link:
-        common.error(
+        log.error(
           f'Link {link._linkname} is not a group link, but has a "members" list',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'links')
       continue
 
@@ -916,15 +975,15 @@ def expand_groups(topology: Box) -> None:
         parent=link,
         key='group',
         path=link._linkname,
-        abort=True,
+        _abort=True,
         module='links')
     except:                                         # If not, report error and skip the link
       continue
 
     if not must_be_list(parent=link,key='members',path=link._linkname,create_empty=False,module='links'):
-      common.error(                                 # Make sure 'members' is a list
+      log.error(                                 # Make sure 'members' is a list
         f'Group link {link._linkname} has no members',
-        common.MissingValue,
+        log.MissingValue,
         'links')
       continue                                      # Report error and skip otherwise
 
