@@ -8,9 +8,10 @@ import argparse
 import os
 import glob
 
-from . import common_parse_args
+from . import parser_add_verbose
+from .external_commands import set_ansible_flags
 from . import ansible
-from .. import common
+from ..utils import log
 
 #
 # CLI parser for 'netlab config' command
@@ -21,21 +22,21 @@ def custom_config_parse(args: typing.List[str]) -> typing.Tuple[argparse.Namespa
     description='Deploy custom configuration template',
     epilog='All other arguments are passed directly to ansible-playbook')
   parser.add_argument(
-    '-v','--verbose',
-    dest='verbose',
+    '-r','--reload',
+    dest='reload',
     action='store_true',
-    help='Verbose logging')
+    help='Reload saved device configurations')
   parser.add_argument(
     dest='template', action='store',
-    help='Configuration template (or a family of templates)')
+    help='Configuration template or a directory with templates')
+  parser_add_verbose(parser)
 
   return parser.parse_known_args(args)
 
 def run(cli_args: typing.List[str]) -> None:
   (args,rest) = custom_config_parse(cli_args)
-
-  if args.verbose:
-    rest = ['-v'] + rest
+  log.set_logging_flags(args)
+  set_ansible_flags(rest)
 
   if args.template != '-':
     if os.path.exists(args.template) or \
@@ -43,8 +44,11 @@ def run(cli_args: typing.List[str]) -> None:
        glob.glob(args.template+'.*.j2'):
       rest = ['-e','config='+args.template] + rest
     else:
-      common.fatal('Cannot find specified Jinja2 template: %s' % args.template,'config')
+      log.fatal(f'Cannot find specified Jinja2 template or configuration directory { args.template }','config')
 
   if args.verbose:
-    print('Ansible playbook args: %s' % rest)
-  ansible.playbook('config.ansible',rest)
+    print(f'Ansible playbook args: { rest }')
+  if args.reload:
+    ansible.playbook('reload-config.ansible',rest)
+  else:
+    ansible.playbook('config.ansible',rest)
