@@ -7,10 +7,13 @@
 import argparse
 import typing
 import textwrap
+import os
+import sys
 from box import Box
 
 from . import common_parse_args, topology_parse_args, load_topology
-from .. import read_topology,augment,common
+from .. import augment
+from ..utils import log, read as _read,strings
 from ..outputs import _TopologyOutput
 
 #
@@ -43,6 +46,8 @@ def create_topology_parse(
     prog=f"netlab {cmd}",
     description=description,
     epilog=epilog)
+  parser.add_argument('--unlock', dest='unlock', action='store_true',
+                  help=argparse.SUPPRESS)
 
   parser.add_argument(
     dest='topology', action='store', nargs='?',
@@ -67,20 +72,25 @@ def run(cli_args: typing.List[str],
     args.devices = None
 
   if not args.output:
-    args.output = ['provider','yaml=netlab.snapshot.yml']
+    args.output = ['provider','yaml=netlab.snapshot.yml','tools']
     args.output.append('devices' if args.devices else 'ansible:dirs')
   elif args.devices:
-    common.error('--output and --devices flags are mutually exclusive',common.IncorrectValue,'create')
+    log.error('--output and --devices flags are mutually exclusive',log.IncorrectValue,'create')
 
   topology = load_topology(args)
   augment.main.transform(topology)
-  common.exit_on_error()
+  log.exit_on_error()
+
+  if args.unlock and os.path.exists('netlab.lock'):
+    strings.print_colored_text("WARNING: ","bright_red",stderr=True)
+    print("removing netlab.lock file, you're on your own",file=sys.stderr)
+    os.remove('netlab.lock')
 
   for output_format in args.output:
     output_module = _TopologyOutput.load(output_format,topology.defaults.outputs[output_format.split(':')[0]])
     if output_module:
       output_module.write(topology)
     else:
-      common.error('Unknown output format %s' % output_format,common.IncorrectValue,'create')
+      log.error('Unknown output format %s' % output_format,log.IncorrectValue,'create')
 
   return topology
