@@ -8,22 +8,18 @@
 import typing
 import argparse
 import os
-import glob
-import subprocess
-import shutil
+import sys
 
 from box import Box
 from pathlib import Path
 
-from .. import common
 from . import create
-from . import external_commands
+from . import external_commands, set_dry_run, is_dry_run
 from . import common_parse_args, get_message
-from . import lab_status_update, lab_status_change, get_lab_id
+from . import lab_status_update, lab_status_change
 from .. import providers
-from .. import read_topology
-from ..utils import status
-from ..data import get_box,get_from_box
+from ..utils import log,strings,status as _status, read as _read
+from ..data import global_vars
 
 #
 # Extra arguments for 'netlab up' command
@@ -39,6 +35,21 @@ def up_parse_args(standalone: bool) -> argparse.ArgumentParser:
     dest='no_config',
     action='store_true',
     help='Do not configure lab devices')
+  parser.add_argument(
+    '-r','--reload-config',
+    dest='reload',
+    action='store',
+    help='Reload saved configurations from specified directory')
+  parser.add_argument(
+    '--no-tools',
+    dest='no_tools',
+    action='store_true',
+    help='Do not start the external tools')
+  parser.add_argument(
+    '--dry-run',
+    dest='dry_run',
+    action='store_true',
+    help='Print the commands that would be executed, but do not execute them')
   parser.add_argument(
     '--fast-config',
     dest='fast_config',
@@ -51,6 +62,11 @@ def up_parse_args(standalone: bool) -> argparse.ArgumentParser:
     nargs='?',
     const='netlab.snapshot.yml',
     help='Use netlab snapshot file created by a previous lab run')
+  parser.add_argument(
+    '--validate',
+    dest='validate',
+    action='store_true',
+    help=argparse.SUPPRESS)
   return parser
 
 """
@@ -61,15 +77,16 @@ def get_topology(args: argparse.Namespace, cli_args: typing.List[str]) -> Box:
 
   if args.snapshot:                                           # If we're using the snapshot file...
     args = up_args_parser.parse_args(cli_args)                # ... and reparse
-    common.set_logging_flags(args)                            # ... use these arguments to set logging flags and read the snapshot
+    log.set_logging_flags(args)                               # ... use these arguments to set logging flags and read the snapshot
 
-    topology = read_topology.read_yaml(filename=args.snapshot)
+    topology = _read.read_yaml(filename=args.snapshot)
     if topology is None:
-      common.fatal(f'Cannot read snapshot file {args.snapshot}, aborting...')
-      return Box({})                                          # pragma: no-cover
+      log.fatal(f'Cannot read snapshot file {args.snapshot}, aborting...')
 
     print(f"Using transformed lab topology from snapshot file {args.snapshot}")
+    global_vars.init(topology)    
   else:                                                       # No snapshot file, use 'netlab create' parser
+    log.section_header('Creating','configuration files')
     topology = create.run(cli_args,'up','Create configuration files, start a virtual lab, and configure it',up_args_parser)
 
   return topology
@@ -84,7 +101,7 @@ Lab status routines:
 """
 
 def lab_status_start(status: Box, topology: Box) -> None:
-  lab_id = get_lab_id(topology)                             # Get the lab ID (or default)
+  lab_id = _status.get_lab_id(topology)                     # Get the lab ID (or default)
   if lab_id in status:
     if status[lab_id].dir != os.getcwd():
       lab_status_update(topology,status,
@@ -99,12 +116,12 @@ def lab_status_start(status: Box, topology: Box) -> None:
       'providers': [] })
 
 def status_start_lab(topology: Box) -> None:
-  status.change_status(
+  _status.change_status(
     topology,
     callback = lambda s,t: lab_status_start(s,t))
 
 def status_start_provider(topology: Box, provider: str) -> None:
-  status.change_status(
+  _status.change_status(
     topology,
     callback = lambda s,t: 
       lab_status_update(t,s,
@@ -115,7 +132,7 @@ def status_start_provider(topology: Box, provider: str) -> None:
 check_existing_lab -- print an command-specific error message if there'a s lab already running in this directory
 """
 def check_existing_lab() -> None:
-  if not status.is_directory_locked():
+  if not _status.is_directory_locked():
     return
   
   print(f'''
@@ -129,14 +146,14 @@ Otherwise use 'netlab status' to check the status of labs running on this machin
 If you are sure that no other lab is running in this directory, remove the
 netlab.lock file manually and retry.
 ''')
-  common.fatal('Cannot start another lab in the same directory')
+  log.fatal('Cannot start another lab in the same directory')
 
 """
 check_lab_instance -- print an error message if the lab instance is already running in a different directory
 """
 def check_lab_instance(topology: Box) -> None:
-  lab_id = get_lab_id(topology)                   # Get the current lab instance ID from lab topology
-  lab_states = status.read_status(topology)       # Read the state of existing lab instances
+  lab_id = _status.get_lab_id(topology)           # Get the current lab instance ID from lab topology
+  lab_states = _status.read_status(topology)      # Read the state of existing lab instances
 
   if not lab_id in lab_states:                    # If this lab instance is not running ==> OK
     return
@@ -154,7 +171,7 @@ You can stop the other lab instance with 'netlab status cleanup {lab_id}'.
 If you think your netlab status file is corrupt, use 'netlab status reset' to
 delete it.
 ''')
-  common.fatal(f'aborting "netlab up" request')
+  log.fatal(f'aborting "netlab up" request')
 
 """
 Execute provider probes
@@ -162,9 +179,10 @@ Execute provider probes
 def provider_probes(topology: Box) -> None:
   p_provider = topology.provider
 
-  external_commands.run_probes(topology.defaults,p_provider,2)
+  log.section_header('Checking','virtualization provider installation')
+  external_commands.run_probes(topology.defaults,p_provider)
   for s_provider in topology[p_provider].providers:
-    external_commands.run_probes(topology.defaults,s_provider,2)
+    external_commands.run_probes(topology.defaults,s_provider)
 
 """
 Start lab topology for a single provider
@@ -187,7 +205,10 @@ def start_provider_lab(topology: Box, pname: str, sname: typing.Optional[str] =
 
   exec_list = exec_command if isinstance(exec_command,list) else [ exec_command ]
   for cmd in exec_list:
-    external_commands.start_lab(topology.defaults,sname or pname,3,"netlab up",cmd)
+    print(f"provider {p_name}: executing {cmd}")
+    if not external_commands.run_command(cmd):
+      log.fatal(f"{cmd} failed, aborting...","netlab up")
+
   p_module.call('post_start_lab',p_topology)
 
   lab_status_change(topology,f'{p_name} workload started')
@@ -206,22 +227,90 @@ def recreate_secondary_config(topology: Box, p_provider: str, s_provider: str) -
   print(f"Recreating {filename} configuration file for {s_provider} provider")
   sp_module.create(s_topology,filename)                               # ... and create the new configuration file
 
+"""
+Deploy initial configuration
+"""
+def deploy_initial_config(args: argparse.Namespace, topology: Box) -> None:
+  if args.no_config:
+    print()
+    strings.print_colored_text('[SKIPPED] ','yellow',None)
+    print("Initial configuration skipped, run 'netlab initial' to configure the devices")
+    return
+
+  lab_status_change(topology,f'deploying initial configuration')
+  log.section_header('Deploying','initial device configurations')
+  external_commands.deploy_configs("netlab up",args.fast_config)
+  lab_status_change(topology,f'initial configuration complete')
+
+  message = get_message(topology,'initial',True)
+  if message:
+    print(f"\n\n{message}")
+
+"""
+Reload saved configurations
+"""
+def reload_saved_config(args: argparse.Namespace, topology: Box) -> None:
+  lab_status_change(topology,f'reloading saved initial configurations')
+  log.section_header('Reloading','saved initial device configurations')
+  cmd = external_commands.set_ansible_flags(['netlab','config','--reload',args.reload])
+  if not external_commands.run_command(cmd):
+    log.fatal("netlab config --reload failed, aborting...",'netlab up')
+  lab_status_change(topology,f'saved initial configurations reloaded')
+  log.status_success()
+  print("Saved configurations reloaded")
+
+"""
+Deploy external tools
+"""
+def start_external_tools(args: argparse.Namespace, topology: Box) -> None:
+  if not 'tools' in topology:
+    return
+  if args.no_tools:
+    print()
+    strings.print_colored_text('[SKIPPED] ','yellow',None)
+    print("External tools not started, start them manually")
+    return
+
+  lab_status_change(topology,f'starting external tools')
+  log.section_header('Starting','external tools')
+  for tool in topology.tools.keys():
+    cmds = external_commands.get_tool_command(tool,'up',topology)
+    if cmds is None:
+      continue
+    external_commands.execute_tool_commands(cmds,topology)
+    msg = external_commands.get_tool_message(tool,topology)
+    if not is_dry_run():
+      print(f"... {tool} tool started")
+
+    if msg:
+      print(("DRY_RUN: " if is_dry_run() else "") + msg + "\n")
+
+  lab_status_change(topology,f'external tools started')
+  log.status_success()
+  print("External tools started")
+
 """
 Main "lab start" process
 """
 def run(cli_args: typing.List[str]) -> None:
   up_args_parser = up_parse_args(False)                       # Try to parse the up-specific arguments
   (args,rest) = up_args_parser.parse_known_args(cli_args)
-  if not args.snapshot:
+  if args.reload and args.no_config:
+    log.fatal('Cannot combine --reload-config and --no-config')
+
+  set_dry_run(args)
+  if not args.snapshot and not is_dry_run():
     check_existing_lab()
 
   topology = get_topology(args,cli_args)
-  check_lab_instance(topology)
+  if not is_dry_run():
+    check_lab_instance(topology)
 
   settings = topology.defaults
-  if common.QUIET:
+  if log.QUIET:
     os.environ["ANSIBLE_STDOUT_CALLBACK"] = "selective"
 
+  external_commands.LOG_COMMANDS = True
   provider_probes(topology)
 
   p_provider = topology.provider
@@ -231,25 +320,32 @@ def run(cli_args: typing.List[str]) -> None:
 
   status_start_lab(topology)
   if 'err_conflict' in topology.defaults:
-    common.fatal(f'race condition, lab instance already running in {topology.defaults.err_conflict}')
-    return
+    log.fatal(f'race condition, lab instance already running in {topology.defaults.err_conflict}')
 
-  status.lock_directory()
+  if not is_dry_run():
+    _status.lock_directory()
 
+  log.section_header('Starting',f'{p_provider} nodes')
   start_provider_lab(topology,p_provider)
+
   for s_provider in topology[p_provider].providers:
-    print()
+    log.section_header('Starting',f'{s_provider} nodes')
     recreate_secondary_config(topology,p_provider,s_provider)
     start_provider_lab(topology,p_provider,s_provider)
 
-  if not args.no_config:
-    lab_status_change(topology,f'deploying initial configuration')
-    external_commands.deploy_configs(4,"netlab up",args.fast_config)
-    message = get_message(topology,'up',False)
-    if message:
-      print(f"\n\n{message}")
-    lab_status_change(topology,f'initial configuration complete')
+  if args.reload:
+    reload_saved_config(args,topology)
   else:
-    print("\nInitial configuration skipped, run 'netlab initial' to configure the devices")
-
+    deploy_initial_config(args,topology)
+  start_external_tools(args,topology)
   lab_status_change(topology,'started')
+  if _status.is_directory_locked():                   # If we're using the lock file, touch it after we're done
+    _status.lock_directory()                          # .. to have a timestamp of when the lab was started
+
+  log.repeat_warnings('netlab up')
+
+  if args.validate:
+    if args.no_config:
+      log.error('Lab is not configured, skipping the validation phase',Warning,'')
+    else:
+      external_commands.run_command('netlab validate')
