@@ -4,8 +4,7 @@
 from box import Box
 
 from . import _Quirks
-from .. import common
-from ..data import get_from_box
+from ..utils import log
 
 def check_isis_p2p_interfaces(node: Box, topology: Box) -> None:
   for intf in node.interfaces:
@@ -16,12 +15,12 @@ def check_isis_p2p_interfaces(node: Box, topology: Box) -> None:
       remote_interfaces = topology.nodes[remote_node].interfaces
       for rintf in remote_interfaces:
         if rintf.ifname == neighbor.ifname:
-          if get_from_box(rintf, 'isis.network_type') == "point-to-point":
-            common.error(
-                f'Cisco ASA does not support P2P IS-IS links.'
-                f'Problematic Interface: {remote_node} {neighbor.ifname}',
-                common.IncorrectType,
-                'quirks',
+          if intf.get('isis.network_type',None) == "point-to-point":
+            log.error(
+              f'Cisco ASA does not support P2P IS-IS links.'
+              f'Problematic Interface: {remote_node} {neighbor.ifname}',
+              log.IncorrectType,
+              'quirks',
             )
 
 class ASA(_Quirks):
