@@ -1,33 +1,36 @@
 #
-# Arista EOS quirks
+# Cisco IOSv quirks
 #
 from box import Box
 
 from . import _Quirks
-from .. import common
-from ..data import get_from_box
+from ..utils import log
+from ..modules import _routing
 from ..augment import devices
 
 # Cisco IOSv does not support VRRP on BVI interfaces. Go figure...
 #
 def check_vrrp_bvi(node: Box, topology: Box) -> None:
   for intf in node.interfaces:
-    if get_from_box(intf,'gateway.protocol') != 'vrrp':                 # No VRRP, move on
+    if intf.get('gateway.protocol',None) != 'vrrp':                     # No VRRP, move on
       continue
 
-    if get_from_box(intf,'type') != 'svi':                              # Not a BVI interface, move on
+    if intf.get('type',None) != 'svi':                                  # Not a BVI interface, move on
       continue
 
-    common.error(
+    log.error(
       f'Cisco IOSv cannot run VRRP on BVI interfaces.',
-      common.IncorrectType,
+      log.IncorrectType,
       'quirks')
     return
 
-class EOS(_Quirks):
+class IOS(_Quirks):
 
   @classmethod
   def device_quirks(self, node: Box, topology: Box) -> None:
     mods = node.get('module',[])
     if 'gateway' in mods:
       check_vrrp_bvi(node,topology)
+
+    if 'ospf' in mods:
+      _routing.get_unique_router_ids(node,'ospf',topology)
