@@ -1,4 +1,8 @@
 #!/bin/bash
+
+# Install a specific version of Containerlab
+CONTAINERLAB_VERSION="0.49.0"
+
 cat <<EOM
 Docker/Containerlab Installation Script
 =====================================================================
@@ -12,15 +16,17 @@ during the installation process.
 
 EOM
 
-# Add sudo / root check - ghostinthenet 20220418
-SUDO='DEBIAN_FRONTEND=noninteractive NEEDRESTART_MODE=a'
-if [ "$UID" != "0" ]; then
- if [ -x "$(command -v sudo)" ]; then
-  SUDO="sudo $SUDO"
- else
-  echo 'Script requires root privileges.'
-  exit 0
- fi
+# If we have sudo command, then we use it to set environment variables
+if [ -x "$(command -v sudo)" ]; then
+  SUDO='sudo DEBIAN_FRONTEND=noninteractive NEEDRESTART_MODE=a'
+else
+#
+# no sudo command, if we're not root we can't proceed
+  SUDO=""
+  if [ "$UID" != "0" ]; then
+    echo 'Script requires root privileges.'
+    exit 0
+  fi
 fi
 
 if [[ -z "$FLAG_YES" ]]; then
@@ -78,8 +84,8 @@ echo "deb [arch=$(dpkg --print-architecture)] https://download.docker.com/linux/
 echo "Install Docker Engine"
 $SUDO apt-get update
 $SUDO apt-get install -y $FLAG_QUIET docker-ce docker-ce-cli containerd.io
-echo "Install containerlab"
-$SUDO bash "-c" "$(curl -sL https://get.containerlab.dev)"
+echo "Install containerlab version $CONTAINERLAB_VERSION"
+$SUDO bash "-c" "$(curl -sL https://get.containerlab.dev)" -- -v $CONTAINERLAB_VERSION
 set +e
 G="$(groups $USER|grep docker)"
 set -e
