@@ -13,15 +13,17 @@ during the installation process.
 
 EOM
 
-# Add sudo / root check - ghostinthenet 20220418
-SUDO='DEBIAN_FRONTEND=noninteractive NEEDRESTART_MODE=a'
-if [ "$UID" != "0" ]; then
- if [ -x "$(command -v sudo)" ]; then
-  SUDO="sudo $SUDO"
- else
-  echo 'Script requires root privileges.'
-  exit 0
- fi
+# If we have sudo command, then we use it to set environment variables
+if [ -x "$(command -v sudo)" ]; then
+  SUDO='sudo DEBIAN_FRONTEND=noninteractive NEEDRESTART_MODE=a'
+else
+#
+# no sudo command, if we're not root we can't proceed
+  SUDO=""
+  if [ "$UID" != "0" ]; then
+    echo 'Script requires root privileges.'
+    exit 0
+  fi
 fi
 
 if [[ -z "$FLAG_YES" ]]; then
@@ -62,8 +64,8 @@ set -e
 curl -fsSL https://apt.releases.hashicorp.com/gpg | $SUDO gpg --dearmor -o /etc/apt/trusted.gpg.d/hashicorp-security.gpg
 $SUDO sh -c 'echo "deb [arch=amd64] https://apt.releases.hashicorp.com $(lsb_release -cs) main" > /etc/apt/sources.list.d/vagrant.list'
 $SUDO apt-get update
-$SUDO apt-get install -y $FLAG_QUIET ruby-dev ruby-libvirt vagrant
-vagrant plugin install vagrant-libvirt --plugin-version=0.11.2
+$SUDO apt-get install -y $FLAG_QUIET ruby-dev ruby-libvirt vagrant=2.4.0-1
+vagrant plugin install vagrant-libvirt --plugin-version=0.12.2
 echo ".. vagrant installed"
 echo
 set +e
