@@ -12,15 +12,17 @@ during the installation process.
 
 EOM
 
-# Add sudo / root check - ghostinthenet 20220418
-SUDO='DEBIAN_FRONTEND=noninteractive NEEDRESTART_MODE=a'
-if [ "$UID" != "0" ]; then
- if [ -x "$(command -v sudo)" ]; then
-  SUDO="sudo $SUDO"
- else
-  echo 'Script requires root privileges.'
-  exit 0
- fi
+# If we have sudo command, then we use it to set environment variables
+if [ -x "$(command -v sudo)" ]; then
+  SUDO='sudo DEBIAN_FRONTEND=noninteractive NEEDRESTART_MODE=a'
+else
+#
+# no sudo command, if we're not root we can't proceed
+  SUDO=""
+  if [ "$UID" != "0" ]; then
+    echo 'Script requires root privileges.'
+    exit 0
+  fi
 fi
 
 if [[ -z "$FLAG_YES" ]]; then
@@ -45,8 +47,16 @@ $SUDO apt-get upgrade -y $FLAG_APT
 # Install missing packages
 #
 echo "Install missing packages (also a pretty long operation)"
+# Add linux-modules-extra for e.g. vrf
+# linux-modules-extra-$(uname -r) should exist, but it won't if we're running
+# on a kernel that has been purged from the package server; in that case the
+# user will likely need to reboot.
+EXTRA=""
+if apt-cache show linux-modules-extra-$(uname -r) > /dev/null; then
+  EXTRA="linux-modules-extra-$(uname -r)"
+fi
 # Added curl, which is not installed by default on Debian - ghostinthenet - 20220417
-$SUDO apt-get -y $FLAG_APT install python3 python3-setuptools ifupdown python3-pip curl
+$SUDO apt-get -y $FLAG_APT install python3 python3-setuptools ifupdown python3-pip curl $EXTRA
 echo "Install nice-to-have packages"
 $SUDO apt-get -y $FLAG_APT install git ack-grep jq tree sshpass colordiff
 #
