@@ -8,22 +8,23 @@ from box import Box
 import netaddr
 
 from . import _Module,_routing
-from .. import common
 from .. import data
-from ..data.validate import must_be_int,must_be_list,must_be_dict
+from ..data.types import must_be_int,must_be_list,must_be_asn
+from ..data.validate import validate_item
 from ..augment import devices
+from ..utils import log
 
-def check_bgp_parameters(node: Box) -> None:
+def check_bgp_parameters(node: Box, topology: Box) -> None:
   if not "bgp" in node:  # pragma: no cover (should have been tested and reported by the caller)
     return
   if not "as" in node.bgp:
-    common.error(
+    log.error(
       f"Node {node.name} has BGP enabled but no AS number specified",
-      common.MissingValue,
+      log.MissingValue,
       'bgp')
     return
 
-  must_be_int(parent=node,key='bgp.as',path=f'nodes.{node.name}',min_value=1,max_value=65535,module='bgp')
+  must_be_asn(parent=node,key='bgp.as',path=f'nodes.{node.name}',module='bgp')
 
   if "community" in node.bgp:
     bgp_comm = node.bgp.community
@@ -32,21 +33,24 @@ def check_bgp_parameters(node: Box) -> None:
     elif isinstance(bgp_comm,list):
       node.bgp.community = { 'ibgp' : bgp_comm, 'ebgp': bgp_comm }
     elif not(isinstance(bgp_comm,dict)):
-      common.error(
+      log.error(
         f"bgp.community attribute in node {node.name} should be a string, a list, or a dictionary (found: {bgp_comm})",
-        common.IncorrectType,
+        log.IncorrectType,
         'bgp')
       return
 
     for k in node.bgp.community.keys():
       if not k in ['ibgp','ebgp']:
-        common.error("Invalid BGP community setting in node %s: %s" % (node.name,k))
+        log.error(
+          text=f"Invalid BGP community setting in {k} node {node.name}",
+          category=log.IncorrectValue,
+          module='bgp')
       else:
         must_be_list(
           parent=node.bgp.community,
           path=f'nodes.{node.name}.bgp.community',
           key=k,
-          valid_values=['standard','extended'],
+          valid_values=topology['defaults.bgp.attributes.global.community.ibgp'],
           module='bgp')
 
 BGP_VALID_AF: typing.Final[list] = ['ipv4','ipv6']
@@ -56,9 +60,9 @@ def validate_bgp_sessions(node: Box, sessions: Box, attribute: str) -> bool:
   OK = True
   for k in list(sessions.keys()):
     if not k in BGP_VALID_AF:
-      common.error(
+      log.error(
         f'Invalid address family in bgp.{attribute} in node {node.name}',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'bgp')
       OK = False
     else:
@@ -92,7 +96,7 @@ bgp_neighbor: Create BGP neighbor data structure
 * extra_data - anything else we might want to pass to the neighbor data structure
 """
 def bgp_neighbor(n: Box, intf: Box, ctype: str, sessions: Box, extra_data: typing.Optional[dict] = None) -> typing.Optional[Box]:
-  ngb = Box(extra_data or {},default_box=True,box_dots=True)
+  ngb = data.get_box(extra_data or {})
   ngb.name = n.name
   ngb["as"] = n.bgp.get("as")
   ngb["type"] = ctype
@@ -122,6 +126,22 @@ def get_neighbor_rr(n: Box) -> typing.Optional[typing.Dict]:
 
   return {}
 
+"""
+get_remote_ibgp_endpoint: find the remote endpoint of an IBGP session
+
+* Loopback interface if it exists
+* First physical interface if the remote device is a daemon without a loopback interface
+* Otherwise, an empty box (which will result in no IBGP session due to lack of IP addresses)
+"""
+def get_remote_ibgp_endpoint(n: Box) -> Box:
+  if 'loopback' in n:
+    return n.loopback
+
+  if n.get('_daemon',False) and n.interfaces:
+    return n.interfaces[0]
+
+  return data.get_empty_box()
+
 """
 build_ibgp_sessions: create IBGP session data structure
 
@@ -130,6 +150,7 @@ build_ibgp_sessions: create IBGP session data structure
 """
 def build_ibgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
   rrlist = find_bgp_rr(node.bgp.get("as"),topology)
+  has_ibgp     = False                            # Assume we have no IBGP sessions (yet)
 
   # If we don't have route reflectors, or if the current node is a route
   # reflector, we need BGP sessions to all other nodes in the same AS
@@ -137,9 +158,11 @@ def build_ibgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
     for name,n in topology.nodes.items():
       if "bgp" in n:
         if n.bgp.get("as") == node.bgp.get("as") and n.name != node.name:
-          neighbor_data = bgp_neighbor(n,n.loopback,'ibgp',sessions,get_neighbor_rr(n))
+          n_intf = get_remote_ibgp_endpoint(n)
+          neighbor_data = bgp_neighbor(n,n_intf,'ibgp',sessions,get_neighbor_rr(n))
           if not neighbor_data is None:
             node.bgp.neighbors.append(neighbor_data)
+            has_ibgp = True
 
   #
   # The node is not a route reflector, and we have a non-empty RR list
@@ -147,9 +170,34 @@ def build_ibgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
   else:
     for n in rrlist:
       if n.name != node.name:
-        neighbor_data = bgp_neighbor(n,n.loopback,'ibgp',sessions,get_neighbor_rr(n))
+        n_intf = get_remote_ibgp_endpoint(n)
+        neighbor_data = bgp_neighbor(n,n_intf,'ibgp',sessions,get_neighbor_rr(n))
         if not neighbor_data is None:
           node.bgp.neighbors.append(neighbor_data)
+          has_ibgp = True
+
+  if not has_ibgp:
+    return
+
+  # Do we have to warn the user that IBGP sessions work better with IGP?
+  # The warning could be disabled in the settings, and we assume it doesn't make
+  # sense complaining if the node has no loopback interfaces (routing on hosts)
+  #
+  ibgp_warning = topology.defaults.bgp.warnings.missing_igp and 'loopback' in node
+  igp_list     = topology.defaults.bgp.warnings.igp_list
+
+  if ibgp_warning:                                # Does the user want to get the warning?
+    for igp in igp_list:                          # If so, check the viable IGPs
+      if igp in node.module:                      # ... and if any one of them is in the list of node modules
+        ibgp_warning = False                      # ... life is good, turn off the warning
+        break
+
+  if ibgp_warning:
+    log.error(
+      f'Node {node.name} has IBGP sessions but no IGP',
+      category=Warning,
+      module='bgp',
+      hint='igp')
 
 """
 build_ebgp_sessions: create EBGP session data structure
@@ -168,7 +216,7 @@ def build_ebgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
       continue                                                        # ... and skip interfaces with 'bgp: False'
 
     node_as =  node.bgp.get("as")                                     # Get our real AS number and the AS number of the peering session
-    node_local_as = data.get_from_box(l,'bgp.local_as') or data.get_from_box(node,'bgp.local_as') or node_as
+    node_local_as = l.get('bgp.local_as',None) or node.get('bgp.local_as',None) or node_as
 
     af_list = [ af for af in ('ipv4','ipv6','unnumbered') if af in l ]
     if not af_list:                                                   # This interface has no usable address
@@ -176,15 +224,15 @@ def build_ebgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
 
     if node_as != node_local_as:
       if not features.bgp.local_as:
-        common.error(
+        log.error(
           text=f'{node.name} (device {node.device}) does not support BGP local AS (interface {l.name})',
-          category=common.IncorrectValue,
+          category=log.IncorrectValue,
           module='bgp')
         continue
       if l.get('vrf',None) and not features.bgp.vrf_local_as:
-        common.error(
+        log.error(
           text=f'{node.name} (device {node.device}) does not support BGP local AS for EBGP sessions in VRF {l.vrf}',
-          category=common.IncorrectValue,
+          category=log.IncorrectValue,
           module='bgp')
         continue
 
@@ -197,8 +245,13 @@ def build_ebgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
       #
       ngb_name = ngb_ifdata.node
       neighbor = topology.nodes[ngb_name]
-      neighbor_real_as = data.get_from_box(neighbor,'bgp.as')
-      neighbor_local_as = data.get_from_box(ngb_ifdata,'bgp.local_as') or data.get_from_box(neighbor,'bgp.local_as') or neighbor_real_as
+      neighbor_real_as = neighbor.get('bgp.as',None)
+      try:                                                            # Try to get neighbor local_as
+        neighbor_local_as = ( ngb_ifdata.get('bgp.local_as',None) or
+                              neighbor.get('bgp.local_as',None) or
+                              neighbor_real_as )
+      except:                                                         # Neighbor could have bgp set to False
+        neighbor_local_as = neighbor_real_as
 
       if not neighbor_local_as:                                       # Neighbor has no usable BGP AS number, move on
         continue
@@ -206,8 +259,10 @@ def build_ebgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
       if node_as == neighbor_real_as and node_local_as == neighbor_local_as:
         continue                                                      # Routers in the same AS + no local-as trickery => nothing to do here
 
-      extra_data = Box({})
+      extra_data = data.get_empty_box()
       extra_data.ifindex = l.ifindex
+      if 'bgp' in ngb_ifdata and isinstance(ngb_ifdata.bgp,Box):      # Copy neighbor BGP interface attributes into neighbor data
+        extra_data = ngb_ifdata.bgp + extra_data                      # ... useful for things like BGP roles
 
       # Figure out whether both neighbors have IPv6 LLA and/or unnumbered IPv4 interfaces
       #
@@ -223,9 +278,9 @@ def build_ebgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
       if ipv6_lla or rfc8950:
         extra_data.local_if = l.ifname                                # Set local_if to indicate IPv6 LLA EBGP session
         if not features.bgp.ipv6_lla:                                 # IPv6_LLA feature flag has to be set even for IPv4 unnumbered EBGP
-          common.error(
+          log.error(
             text=f'{node.name} (device {node.device}) does not support EBGP sessions over auto-generated IPv6 LLA (interface {l.name})',
-            category=common.IncorrectValue,
+            category=log.IncorrectValue,
             module='bgp')
           continue
 
@@ -234,23 +289,26 @@ def build_ebgp_sessions(node: Box, sessions: Box, topology: Box) -> None:
         if not 'ipv6' in l:                                           # ... and enable IPv6 on the interface in case a device needs an
           l.ipv6 = True                                               # ... explicit configuration of IPv6 LLA
         if not features.bgp.rfc8950:
-          common.error(
+          log.error(
             text=f'{node.name} (device {node.device}) does not support IPv4 RFC 8950-style AF over IPv6 LLA EBGP sessions (interface {l.name})',
-            category=common.IncorrectValue,
+            category=log.IncorrectValue,
             module='bgp')
           continue
 
-      for k in ('local_as','replace_global_as'):
-        local_as_data = data.get_from_box(l,f'bgp.{k}') or data.get_from_box(node,f'bgp.{k}')
+      for k in ('local_as','replace_global_as'):      # Special handling for local-as attributes
+        extra_data.pop(k,None)                        # These attributes are copied from local data, not neighbor data ==> remove neighbor data
+
+        # Get local settings from link or node
+        local_as_data = l.get(f'bgp.{k}',None) or node.get(f'bgp.{k}',None)
         if not local_as_data is None:
-          extra_data[k] = local_as_data
+          extra_data[k] = local_as_data               # ... and copy it into neighbor data
 
       session_type = 'localas_ibgp' if neighbor_local_as == node_local_as else 'ebgp'
       if session_type == 'localas_ibgp':
         if not features.bgp.local_as_ibgp:
-          common.error(
+          log.error(
             text=f'You cannot use BGP local-as to create an IBGP session with {ngb_name} on {node.name} (device {node.device})',
-            category=common.IncorrectValue,
+            category=log.IncorrectValue,
             module='bgp')
           continue
 
@@ -314,9 +372,9 @@ def build_bgp_rr_clusters(topology: Box) -> None:
         try:
           n.bgp.rr_cluster_id = str(netaddr.IPAddress(n.bgp.rr_cluster_id).ipv4())
         except Exception as ex:
-          common.error(
+          log.error(
             f'BGP cluster ID {n.bgp.rr_cluster_id} configured on node {n.name} cannot be converted into an IPv4 address',
-            common.IncorrectValue,
+            log.IncorrectValue,
             'bgp')
 
 """
@@ -334,12 +392,15 @@ BGP_DEFAULT_SESSIONS: typing.Final[dict] = {
 }
 
 def build_bgp_sessions(node: Box, topology: Box) -> None:
-  if not data.get_from_box(node,'bgp.as'):                        # Sanity check: do we have usable AS number
-    common.fatal(f'build_bgp_sessions: node {node.name} has no usable BGP AS number, how did we get here???')
-    return                                                        # ... not really, get out of here
+  if not isinstance(node.get('bgp',None),Box) or not node.get('bgp.as',None):   # Sanity check
+    log.fatal(
+      f'build_bgp_sessions: node {node.name} has no usable BGP AS number, how did we get here???',
+      module='bgp',
+      header=True)
+    return                                                                      # ... it's insane, get out of here
 
   node.bgp.neighbors = []
-  bgp_sessions = node.bgp.get('sessions') or Box(BGP_DEFAULT_SESSIONS)
+  bgp_sessions = node.bgp.get('sessions') or data.get_box(BGP_DEFAULT_SESSIONS)
   if not validate_bgp_sessions(node,bgp_sessions,'sessions'):
     return
 
@@ -358,13 +419,13 @@ def build_bgp_sessions(node: Box, topology: Box) -> None:
   features = devices.get_device_features(node,topology.defaults)
   if 'activate' in node.bgp:
     if not features.bgp.activate_af:
-      common.error(
+      log.error(
         f'node {node.name} (device {node.device}) does not support configurable activation of default BGP address families',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'bgp')
       return
 
-  activate = node.bgp.get('activate') or Box(BGP_DEFAULT_SESSIONS)
+  activate = node.bgp.get('activate') or data.get_box(BGP_DEFAULT_SESSIONS)
   if not validate_bgp_sessions(node,activate,'activate'):
     return
 
@@ -382,60 +443,92 @@ def bgp_set_advertise(node: Box, topology: Box) -> None:
         continue
       if "advertise" in l.bgp:              # Skip interfaces that already have the 'advertise' flag
         continue
-    if l.get("type",None) in stub_roles or l.get("role",None) in stub_roles:
-      l.bgp.advertise = True                # ... otherwise figure out whether to advertise the subnet
-      continue
+
+    role = l.get("role",None)               # Get interface/link role
+    if l.get("type",None) in stub_roles or role in stub_roles:
+      # We have a potential stub link according to advertised_roles, but we still have to check for true stub
+      #
+      if role != "stub":
+        l.bgp.advertise = True              # No problem if the role is not stub
+      else:                                 # Otherwise figure out whether it's a true stub
+        l.bgp.advertise = _routing.is_true_stub(l,topology)
+      continue                              # And move on
+
     if l.get('type',None) == 'loopback' and node.bgp.advertise_loopback:
       l.bgp.advertise = True                # ... also advertise loopback prefixes if bgp.advertise_loopback is set
 
+"""
+bgp_set_originate_af: set bgp[af] flags based on prefixes that should be originated
+"""
+
+def bgp_set_originate_af(node: Box, topology: Box) -> None:
+  if 'bgp' not in node:                                     # Safeguard: skip non-BGP nodes
+    return
+
+  if node.get('bgp.originate',[]):                          # bgp.originate attribute implies IPv4 is active
+    node.bgp.ipv4 = True
+
+  for af in ['ipv4','ipv6']:
+    if node.get(f'bgp.{af}',False):                         # No need for further checks if the AF flag is already set
+      continue
+
+    if node.get('bgp.advertise_loopback',True):             # If the router advertises its loopback prefix
+      if af in node.get('loopback',{}):                     # ... do the AF checks on loopback interface
+        node.bgp[af] = True
+        continue
+
+    for intf in node.get("interfaces",[]):                  # No decision yet, iterate over all interfaces
+      if af in intf and intf[af]:                           # Is the address family active on the interface?
+        if intf.get('bgp.advertise',False):                 # Is the interface prefix advertised in BGP?
+          if not 'vrf' in intf:                             # The AF fix is only required for global interfaces
+            node.bgp[af] = True                             # ... the stars have aligned, set the BGP AF flag
+
 """
 process_as_list:
   If the global BGP parameters have as_list attribute, set node AS numbers and node
   RR flags accordingly
+
+This function is called from init_groups very early in the topology initialization process
+and has to do its own data validation.
 """
 def process_as_list(topology: Box) -> None:
-  if not data.get_from_box(topology,'bgp.as_list'):         # Do we have global bgp.as_list setting?
-    return                                                  # ... nope, no work for me ;)) 
+  if not topology.get('bgp.as_list'):       # Do we have global bgp.as_list setting?
+    return                                  # ... nope, no work for me ;))
 
-  try:
-    must_be_dict(topology.bgp,'as_list','bgp',create_empty=False,module='bgp',abort=True)
-  except Exception as ex:
+  if not validate_item(
+            parent=topology.bgp,
+            key='as_list',
+            data_type=topology.defaults.bgp.attributes.as_list,
+            parent_path='bgp',
+            data_name='BGP as_list',
+            module='bgp',
+            module_source='topology',
+            topology=topology,
+            attr_list=[ 'bgp' ],
+            attributes=topology.defaults.attributes,
+            enabled_modules=[]):
     return
 
-  node_data = Box({},default_box=True,box_dots=True)
-  node_list = list(topology.nodes.keys())
+  if log.pending_errors():
+    return
+
+  node_data = data.get_empty_box()
   for asn,as_data in topology.bgp.as_list.items():
-    if not isinstance(as_data,Box):
-      common.error(
-        f"Invalid value in bgp.as_list for ASN {asn}\n" + \
-        f"... Each ASN in a BGP as_list must be a dictionary with (at least) members key:\n"+
-        f"... Found: {as_data}",
-        common.IncorrectValue,
-        'bgp')
-      continue
-
-    must_be_list(as_data,'members',f'bgp.as_list.{asn}',create_empty=False,module='bgp',valid_values=node_list)
-    must_be_list(as_data,'rr',f'bgp.as_list.{asn}',create_empty=False,module='bgp',valid_values=node_list)
-    if not as_data.members:
-      common.error(
-        f"BGP as_list for ASN {asn} does not have a valid list of members",
-        common.IncorrectValue,
-        'bgp')
-      continue
-
     for n in as_data.members:
       if 'as' in node_data[n]:
-        common.error(
-          f"BGP module supports at most 1 AS per node; {n} is already member of {node_data[n]['as']} and cannot also be part of {asn}",
-          common.IncorrectValue)
+        log.error(
+          text=f"BGP module supports at most 1 AS per node; {n} is already member of {node_data[n]['as']} and cannot also be part of {asn}",
+          category=log.IncorrectValue,
+          module='bgp')
         continue
       node_data[n]["as"] = asn
 
     for n in as_data.get('rr',{}):
       if node_data[n]["as"] != asn:
-        common.error(
-          "Node %s is specified as route reflector in AS %s but is not in member list" % (n,asn),
-          common.IncorrectValue)
+        log.error(
+          text=f"Node {n} is specified as route reflector in AS {asn} but is not in member list",
+          category=log.IncorrectValue,
+          module='bgp')
         continue
       node_data[n].rr = True
 
@@ -443,13 +536,46 @@ def process_as_list(topology: Box) -> None:
     if name in node_data:
       node_as = node.bgp.get("as",None)
       if node_as and node_as != node_data[name]["as"]:
-        common.error(
-          "Node %s has AS %s but is also in member list of AS %s" % (node.name,node_as,node_data[node.name]["as"]),
-          common.IncorrectValue)
+        log.error(
+          text=f'Node {node.name} has AS {node_as} but is also in member list of AS {node_data[node.name]["as"]}',
+          category=log.IncorrectValue,
+          module='bgp')
         continue
 
       node.bgp = node_data[name] + node.bgp
 
+BGP_DEFAULT_COMMUNITY_KW: typing.Final[dict] = {
+  'standard': 'standard',
+  'extended': 'extended'
+}
+
+"""
+bgp_transform_community_list: transform _netlab_ community keywords into device keywords
+"""
+def bgp_transform_community_list(node: Box, topology: Box) -> None:
+  clist = node.get('bgp.community')
+  if not clist:
+    return
+  
+  features = devices.get_device_features(node,topology.defaults)
+  kw_xform = features.get('bgp.community') or BGP_DEFAULT_COMMUNITY_KW
+
+  for s_type in list(clist.keys()):
+    kw_list = clist[s_type]
+    for kw in kw_list:
+      if not kw in kw_xform:
+        log.error(
+          f"Invalid {s_type} BGP community propagation keyword '{kw}' for device {node.device}/node {node.name}",
+          category=log.IncorrectValue,
+          module='bgp',
+          more_hints=[ f"Valid values are {','.join(kw_xform.keys())}" ])
+    
+  if 'ibgp_localas' not in clist:
+    clist.ibgp_localas = clist.ibgp
+
+  for s_type in list(clist.keys()):
+    clist[s_type] = data.kw_list_transform(kw_xform,clist[s_type])
+
 class BGP(_Module):
 
   """
@@ -471,7 +597,7 @@ class BGP(_Module):
       if node.name in topology.bgp.rr_list:
         node.bgp.rr = True
 
-    check_bgp_parameters(node)
+    check_bgp_parameters(node, topology)
 
   """
   Link pre-transform: Set link role based on BGP nodes attached to the link.
@@ -510,8 +636,11 @@ class BGP(_Module):
   #
   def node_post_transform(self, node: Box, topology: Box) -> None:
     if not "bgp" in node:   # pragma: no cover (this should have been caught in check_bgp_parameters)
-      common.fatal(f"Internal error: node {node.name} has BGP module enabled but no BGP parameters","bgp")
+      log.fatal(f"Internal error: node {node.name} has BGP module enabled but no BGP parameters","bgp")
       return
     build_bgp_sessions(node,topology)
     bgp_set_advertise(node,topology)
+    bgp_set_originate_af(node,topology)
     _routing.remove_vrf_routing_blocks(node,'bgp')
+    bgp_transform_community_list(node,topology)
+    _routing.check_vrf_protocol_support(node,'bgp',None,'bgp',topology)
