@@ -2,7 +2,7 @@ import typing
 
 from . import _Module,_routing,_dataplane
 from box import Box
-from .. import common
+from ..utils import log
 from .. import data
 from ..data.validate import must_be_int,must_be_dict,must_be_string
 from ..augment import devices
@@ -39,11 +39,11 @@ def validate_evpn_lists(toponode: Box, obj_path: str, topology: Box, create: boo
     reference_name='VLAN',
     create_default=create,
     merge_topology=False,
-    default_filter=lambda v: False if not isinstance(v,Box) else data.get_from_box(v,'evpn.transit_vni'),
+    default_filter=lambda v: False if not isinstance(v,Box) else v.get('evpn.transit_vni',False),
     module='evpn')
 
 def enable_evpn_af(node: Box, topology: Box) -> None:
-  bgp_session = data.get_from_box(node,'evpn.session') or []
+  bgp_session = node.get('evpn.session',[])
 
   # Enable EVPN AF on all BGP neighbors with the correct session type
   # that also use EVPN module
@@ -53,17 +53,17 @@ def enable_evpn_af(node: Box, topology: Box) -> None:
       bn.evpn = True
 
 def get_usable_evpn_asn(topology: Box) -> int:
-  asn = data.get_from_box(topology,'evpn.as') or \
-        data.get_from_box(topology,'vrf.as') or \
-        data.get_from_box(topology,'bgp.as')
+  asn = ( topology.get('evpn.as',None) or
+          topology.get('vrf.as',None) or
+          topology.get('bgp.as',None) )
 
   if asn and data.is_true_int(asn):
     return asn
 
-  common.error(
+  log.error(
     f'Cannot get a usable global AS number to use in EVPN route targets',
-    common.IncorrectValue,
-    'evpn')
+    log.IncorrectValue,
+    'evpn',hint='asn')
   return 0
 
 def vlan_based_service(vlan: Box, vname: str, topology: Box) -> None:
@@ -82,16 +82,16 @@ def vlan_based_service(vlan: Box, vname: str, topology: Box) -> None:
 def vlan_aware_bundle_service(vlan: Box, vname: str, topology: Box) -> None:
   vrf_name = vlan.vrf
   if not vrf_name in topology.vrfs:
-    common.error(
+    log.error(
       f'VXLAN-enabled VLAN {vname} that is part of VLAN bundle must belong to a global VRF',
-      common.IncorrectValue,
+      log.IncorrectValue,
       'evpn')
     return
 
   if 'evpn' in vlan:                                                # VLAN that is part of a bundle cannot have EVI/RT/RD attributes
-    common.error(
+    log.error(
       f'VLAN {vname} is part of a VLAN bundle {vrf_name} and cannot have EVPN-related attributes',
-      common.IncorrectValue,
+      log.IncorrectValue,
       'evpn')
     return
 
@@ -124,7 +124,7 @@ def get_vlan_bundle_flag(vlan: Box, vname: str, topology: Box) -> bool:
     return False
 
   vrf = topology.vrfs[vlan.vrf]
-  if not data.get_from_box(vrf,'evpn.bundle'):                      # VRF does not have evpn.bundle attribute ==> VLAN service
+  if not vrf.get('evpn.bundle',None):                               # VRF does not have evpn.bundle attribute ==> VLAN service
     return False
 
   must_be_string(
@@ -155,11 +155,11 @@ def validate_no_node_vrf_bundle(node: Box, topology: Box) -> None:
   if not 'vrfs' in node:                                            # No VRFs defined in the node, move on
     return
   for vname,vdata in node.vrfs.items():
-    if not data.get_from_box(vdata,'evpn.bundle'):                  # EVPN bundle not set in VRF, move on
+    if not vdata.get('evpn.bundle',None):                           # EVPN bundle not set in VRF, move on
       continue
-    common.error(
+    log.error(
       f'VRF {vname} in node {node.name} cannot have evpn.bundle attribute',
-      common.IncorrectValue,
+      log.IncorrectValue,
       'evpn',
       hint='node_bundle')
 
@@ -171,12 +171,12 @@ def register_static_transit_vni(topology: Box) -> None:
   for vrf_name,vrf_data in topology.get('vrfs',{}).items():
     must_be_dict(vrf_data,'evpn',f'vrfs.{vrf_name}',create_empty=False)
 
-    transit_vni = data.get_from_box(vrf_data,'evpn.transit_vni')
+    transit_vni = vrf_data.get('evpn.transit_vni',None)
     if data.is_true_int(transit_vni):
       if transit_vni in vni_set:
-        common.error(
+        log.error(
           f'transit VNI {transit_vni} for VRF {vrf_name} is already used elsewhere',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'evpn')
         continue
       vni_set.add(transit_vni)
@@ -186,10 +186,10 @@ def register_static_transit_vni(topology: Box) -> None:
       continue
 
     for vrf_name,vrf_data in n.vrfs.items():
-      if data.get_from_box(vrf_data,'evpn.transit_vni'):
-        common.error(
+      if vrf_data.get('evpn.transit_vni',None):
+        log.error(
           f'evpn.transit_vni can be specified only on global VRFs (found in {vrf_name} on {n.name}',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'evpn')
 
 """
@@ -208,14 +208,14 @@ def vrf_transit_vni(topology: Box) -> None:
   vni_list: typing.List[int] = []                               # List of static transit VNIs
   vni_error = False                                             # "A horrible error" flag that causes abort after the first loop
   vni_count = 0                                                 # Number of VRFs with evpn.transit_vni
-  evpn_transport = data.get_from_box(topology,'evpn.transport') or 'vxlan'
+  evpn_transport = topology.get('evpn.transport','vxlan')       # Default to VXLAN transport
 
   for vrf_name,vrf_data in topology.vrfs.items():               # First pass: build a list of statically configured VNIs
-    vni = data.get_from_box(vrf_data,'evpn.transit_vni')        # transit_vni makes no sense with MPLS transport
+    vni = vrf_data.get('evpn.transit_vni',None)                 # transit_vni makes no sense with MPLS transport
     if vni and evpn_transport != 'vxlan':
-      common.error(
+      log.error(
         f'evpn.transit_vni in VRF {vrf_name} is not allowed with mpls evpn.transport',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'evpn')
       vni_error = True
       continue
@@ -225,9 +225,9 @@ def vrf_transit_vni(topology: Box) -> None:
     if not data.is_true_int(vni):                               # Skip non-integer values, no need to check them at this time
       continue
     if vni in vni_list:
-      common.error(
+      log.error(
         f'VRF {vrf_name} is using the same EVPN transit VNI as another VRF',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'evpn')
       continue  
     vni_list.append( vni )                                      # Insert it to detect duplicates elsewhere
@@ -239,7 +239,7 @@ def vrf_transit_vni(topology: Box) -> None:
 
   vni_start = topology.defaults.evpn.start_transit_vni
   for vrf_name,vrf_data in topology.vrfs.items():               # Second pass: set transit VNI values for VRFs with "transit_vni: True"
-    if isinstance(data.get_from_box(vrf_data,'evpn.transit_vni'),str):
+    if isinstance(vrf_data.get('evpn.transit_vni',None),str):
       continue                                                  # Skip transit_vni string values (will be checked in third pass)
     transit_vni = must_be_int(
                     vrf_data,
@@ -255,20 +255,20 @@ def vrf_transit_vni(topology: Box) -> None:
   for vrf_name,vrf_data in topology.vrfs.items():               # Third pass: set shared VNI values across VRFs
     if vrf_data is None:                                        # Skip empty VRF definitions
       continue
-    transit_vni = data.get_from_box(vrf_data,'evpn.transit_vni')
+    transit_vni = vrf_data.get('evpn.transit_vni',None)
     if not isinstance(transit_vni,str):                         # Skip if transit_vni is not a string
       continue
     if not transit_vni in topology.vrfs:                        # Does transit VNI refer to a valid VRF name?
-      common.error(
+      log.error(
         f'evpn.transit_vni "{transit_vni}" in VRF {vrf_name} does not refer to a valid VRF',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'evpn')
       continue
-    foreign_vni = data.get_from_box(topology.vrfs,f'{transit_vni}.evpn.transit_vni')
+    foreign_vni = topology.vrfs.get(f'{transit_vni}.evpn.transit_vni',None)
     if not data.is_true_int(foreign_vni):
-      common.error(
+      log.error(
         f'evpn.transit_vni "{transit_vni}" in VRF {vrf_name} refers to a VRF that does not have a valid evpn.transit_vni',
-        common.IncorrectValue,
+        log.IncorrectValue,
         'evpn')
       continue
     vrf_data.evpn.transit_vni = foreign_vni
@@ -313,9 +313,9 @@ def set_local_evpn_rd(node: Box) -> None:
       if not 'evpn' in o_data:
         continue                                                # Skip non-EVPN objects
       if not 'evi' in o_data.evpn:
-        common.error(
+        log.error(
           f'{obj}.{o_name} on node {node.name} has EVPN attributes but no evpn.evi ({o_data.evpn})',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'evpn')
         continue
       #
@@ -341,9 +341,9 @@ def check_asym_vlan(vrf_name: str, node: Box, topology: Box) -> None:
     if vl_name in node.get('vlans',{}):                         # Is the VLAN present on the node?
       continue                                                  # Yeah, everything OK
 
-    common.error(
+    log.error(
       f'VLAN {vl_name} -- part of VRF {vrf_name} that uses asymmetric IRB -- is not present on node {node.name}',
-      common.IncorrectValue,
+      log.IncorrectValue,
       module='evpn',
       hint='irb_group')
 
@@ -352,27 +352,27 @@ Check whether VXLAN IRB mode is supported by the device
 """
 def check_node_vrf_irb(node: Box, topology: Box) -> None:
   features = devices.get_device_features(node,topology.defaults)
-  evpn_transport = data.get_from_box(node,'evpn.transport') or 'vxlan'
+  evpn_transport = node.get('evpn.transport','vxlan')
 
   for vrf_name,vrf_data in node.get('vrfs',{}).items():
     if not vrf_data.get('af',None):                             # Cannot do IRB without L3 addresses ;)
       continue
 
-    symmetric_irb = data.get_from_box(vrf_data,'evpn.transit_vni') or evpn_transport == 'mpls'
+    symmetric_irb = vrf_data.get('evpn.transit_vni',False) or evpn_transport == 'mpls'
     if symmetric_irb:
       if not features.evpn.irb and evpn_transport == 'vxlan':   # ... does this device support IRB?
-        common.error(
+        log.error(
           f'VRF {vrf_name} on {node.name} uses symmetrical EVPN IRB which is not supported by {node.device} device',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'evpn')
         continue
 
       vrf_data.pop('ospf',None)                                 # Remove OSPF from EVPN IRB VRF
     else:
       if not features.evpn.asymmetrical_irb:                    # ... does this device asymmetrical IRB -- is it supported?
-        common.error(
+        log.error(
           f'VRF {vrf_name} on {node.name} uses asymmetrical EVPN IRB which is not supported by {node.device} device',
-          common.IncorrectValue,
+          log.IncorrectValue,
           'evpn')
         continue
 
@@ -385,13 +385,13 @@ Check whether the node supports the requested EVPN bundle type
 def check_node_vrf_bundle(node: Box, topology: Box) -> None:
   features = devices.get_device_features(node,topology.defaults)      # Get device features
   for vrf_name,vrf_data in node.get('vrfs',{}).items():               # Iterate over all VRFs defined on the device
-    b_type = data.get_from_box(vrf_data,'evpn.bundle')                # Get evpn.bundle value, skip if not defined
+    b_type = vrf_data.get('evpn.bundle',None)                         # Get evpn.bundle value, skip if not defined
     if not b_type:
       continue
     if not b_type in features.evpn.bundle:                            # EVPN bundle type not supported by the device
-      common.error(
+      log.error(
         f"'{b_type}'' EVPN bundle service used in VRF {vrf_name} is not supported by device {node.device} (node: {node.name})",
-        common.IncorrectValue,
+        log.IncorrectValue,
         'evpn')
 
 class EVPN(_Module):
@@ -417,7 +417,7 @@ class EVPN(_Module):
       validate_evpn_lists(n,f'nodes.{n.name}',topology,create=False)
 
     vrf_transit_vni(topology)
-    for vname in data.get_from_box(topology,'evpn.vlans') or []:
+    for vname in topology.get('evpn.vlans',[]):
       create_vlan_service(vname,topology)
 
     vrf_irb_setup(topology)
