@@ -4,8 +4,6 @@
 from box import Box
 
 from . import _Module
-from .. import addressing, common
-import netaddr
 
 class SR(_Module):
   pass
