@@ -6,11 +6,14 @@ import typing
 import yaml
 import os
 from box import Box
+import netaddr
 
-from .. import common
 from . import _TopologyOutput,check_writeable
 from ..augment import nodes
 from ..augment import devices
+from ..augment import plugin
+from ..utils import templates,strings,log
+from ..utils import files as _files
 
 forwarded_port_name = { 'ssh': 'ansible_port', }
 
@@ -64,14 +67,50 @@ def ansible_inventory_host(node: Box, topology: Box) -> Box:
   provider_inventory_settings(host,node,topology)
   return host
 
+"""
+Create a 'hosts' dictionary as an 'all' group variable listing usable IPv4 and
+IPv6 addresses of all lab devices.
+"""
+def add_host_addresses(topology: Box, inventory: Box) -> None:
+  for name,node in topology.nodes.items():
+    intf_list = node.interfaces
+    if 'loopback' in node:                                  # Create a list of all usable interfaces
+      intf_list = [ node.loopback ] + node.interfaces       # ... starting with loopback
+
+    for intf in intf_list:                                  # Now iterate over interfaces
+      for af in ('ipv4','ipv6'):                            # ... and collect IPv4 and IPv6 addresses
+        if not isinstance(intf.get(af,False),str):          # Is the IP address a string (= usable IP address)?
+          continue
+      
+        if not af in inventory.all.vars.hosts[name]:        # Edge case: first interface
+          inventory.all.vars.hosts[name][af] = []           # ... have to start with an empty list
+
+        addr = str(netaddr.IPNetwork(intf[af]).ip)          # Extract IP address from the CIDR prefix
+        inventory.all.vars.hosts[name][af].append(addr)     # ... and append it to the list of usable IP addresses
+
+"""
+Copy defaults.paths dictionary into ALL group. Create separate variables
+from individual customizable paths to prevent dependencies on too many
+variables that might not be set (example: only 'paths.custom' values should
+depend on 'custom_config' variable)
+"""
+def copy_paths(inventory: Box, topology: Box) -> None:
+  if 'paths' not in topology.defaults:
+    return
+  
+  for k,v in topology.defaults.paths.items():
+    inventory.all.vars[f'paths_{k}'] = v
+
 def create(topology: Box) -> Box:
   inventory = Box({},default_box=True,box_dots=True)
 
   inventory.all.vars.netlab_provider = topology.defaults.provider
   inventory.all.vars.netlab_name = topology.name
+  copy_paths(inventory,topology)
 
   inventory.modules.hosts = {}
   inventory.custom_configs.hosts = {}
+  inventory.daemons.hosts = {}
 
   defaults = topology.defaults
 
@@ -85,15 +124,21 @@ def create(topology: Box) -> Box:
         if ('_pfx' in k) or ('_eui' in k):
           del pool[k]
 
+  add_host_addresses(topology,inventory)    # Create the 'hosts' dictionary in 'all' group
+
+  extra_groups: dict = {                    # Extra groups created in Ansible inventory
+    'module':  'modules',                   # Devices using configuration modules
+    'config':  'custom_configs',            # Devices using custom configuration
+    '_daemon': 'daemons'                    # Daemons
+  }
+
   for name,node in topology.nodes.items():
     group = node.get('device','all')
     inventory[group].hosts[name] = ansible_inventory_host(node,topology)
 
-    if 'module' in node:
-      inventory.modules.hosts[name] = {}
-
-    if 'config' in node:
-      inventory.custom_configs.hosts[name] = {}
+    for xg in extra_groups.keys():
+      if node.get(xg,False):                # Add device to the extra group if it has the corresponding attribute set
+        inventory[extra_groups[xg]].hosts[name] = {}
 
   if 'devices' in defaults:
     for group in inventory.keys():
@@ -103,6 +148,12 @@ def create(topology: Box) -> Box:
         if group_vars:
           inventory[group]['vars'] = group_vars
 
+  if (inventory.custom_configs.hosts):
+    try:
+      inventory.custom_configs.vars.netlab_custom_config = plugin.sort_extra_config(topology)
+    except log.FatalError as ex:
+      log.fatal(f'Cannot sort custom configuration requests: {str(ex)}','ansible',header=True)
+
   if not 'groups' in topology:
     return inventory
 
@@ -129,17 +180,14 @@ def dump(data: Box) -> None:
   print("Ansible inventory data")
   print("===============================")
   inventory = create(data)
-  print(common.get_yaml_string(inventory))
+  print(strings.get_yaml_string(inventory))
 
 def write_yaml(data: Box, fname: str, header: str) -> None:
   dirname = os.path.dirname(fname)
   if dirname and not os.path.exists(dirname):
     os.makedirs(dirname)
 
-  with open(fname,"w") as output:
-    output.write(header)
-    output.write(common.get_yaml_string(data))
-    output.close()
+  _files.create_file_from_text(fname,header+"\n"+strings.get_yaml_string(data))
 
 min_inventory_data = [ 'id','ansible_host','ansible_port','ansible_connection','ansible_user','ansible_ssh_pass' ]
 
@@ -156,15 +204,17 @@ def ansible_inventory(topology: Box, fname: typing.Optional[str] = 'hosts.yml',
 
   if hostvars == "min":
     write_yaml(inventory,fname,header)
-    print("Created single-file Ansible inventory %s" % fname)
+    log.status_created()
+    print(f"single-file Ansible inventory {fname}")
     return
 
   for g in inventory.keys():
     gvars = inventory[g].pop('vars',None)
     if gvars:
       write_yaml(gvars,'group_vars/%s/topology.yml' % g,header)
-      if not common.QUIET:
-        print("Created group_vars for %s" % g)
+      if not log.QUIET:
+        strings.print_colored_text('[GROUPS]  ','bright_cyan','Created ')
+        print(f"group_vars for {g}")
 
     if 'hosts' in inventory[g]:
       hosts = inventory[g]['hosts']
@@ -180,12 +230,14 @@ def ansible_inventory(topology: Box, fname: typing.Optional[str] = 'hosts.yml',
             vars_host[item] = hosts[h][item]
 
         write_yaml(vars_host,'host_vars/%s/topology.yml' % h,header)
-        if not common.QUIET:
-          print("Created host_vars for %s" % h)
+        if not log.QUIET:
+          strings.print_colored_text('[HOSTS]   ','bright_cyan','Created ')
+          print(f"host_vars for {h}")
         hosts[h] = min_host
 
   write_yaml(inventory,fname,header)
-  print("Created minimized Ansible inventory %s" % fname)
+  log.status_created()
+  print(f"minimized Ansible inventory {fname}")
 
 def ansible_config(config_file: typing.Union[str,None] = 'ansible.cfg', inventory_file: typing.Union[str,None] = 'hosts.yml') -> None:
   if not config_file:
@@ -193,14 +245,26 @@ def ansible_config(config_file: typing.Union[str,None] = 'ansible.cfg', inventor
   if not inventory_file:
     inventory_file = 'hosts.yml'
 
-  with open(config_file,"w") as output:
-    output.write(common.template('ansible.cfg.j2',{ 'inventory': inventory_file or 'hosts.yml' },'templates','ansible'))
-    output.close()
-    if not common.QUIET:
-      print("Created Ansible configuration file: %s" % config_file)
+  try:
+    cfg_text = templates.render_template(
+                j2_file='ansible.cfg.j2',
+                data={'inventory': inventory_file or 'hosts.yml'},
+                path='templates',
+                extra_path=_files.get_search_path('ansible'))
+  except Exception as ex:
+    log.fatal(
+      text=f"Error rendering ansible.cfg\n{strings.extra_data_printout(str(ex))}",
+      module='ansible')
+
+  _files.create_file_from_text(config_file,cfg_text)
+  if not log.QUIET:
+    log.status_created()
+    print(f"Ansible configuration file: {config_file}")
 
 class AnsibleInventory(_TopologyOutput):
 
+  DESCRIPTION :str = 'Ansible inventory and configuration file'
+
   def write(self, topology: Box) -> None:
     check_writeable('Ansible inventory')
     hostfile = self.settings.hostfile or 'hosts.yml'
@@ -212,7 +276,7 @@ class AnsibleInventory(_TopologyOutput):
       if len(self.filenames) > 1:
         configfile = self.filenames[1]
       if len(self.filenames) > 2:
-        common.error('Extra output filename(s) ignored: %s' % str(self.filenames[2:]),common.IncorrectValue,'ansible')
+        log.error('Extra output filename(s) ignored: %s' % str(self.filenames[2:]),log.IncorrectValue,'ansible')
 
     if self.format:
       output_format = self.format[0]
