@@ -6,8 +6,8 @@ from box import Box
 
 from . import _TopologyOutput,check_writeable
 from .. import providers
-from .. import common
 from ..augment import nodes
+from ..utils import log,strings
 
 def get_provider_module(topology: Box, provider: str) -> providers._Provider:
   return providers._Provider.load(provider,topology.defaults.providers[provider])
@@ -18,16 +18,18 @@ def write_provider_file(topology: Box, provider: str, filename: typing.Optional[
 
 class ProviderConfiguration(_TopologyOutput):
 
+  DESCRIPTION :str = 'Create virtualization provider configuration file(s)'
+
   def write(self, topology: Box) -> None:
     check_writeable('provider configuration')
     filename = None
     if hasattr(self,'filenames'):
       filename = self.filenames[0]
       if len(self.filenames) > 1:
-        common.error('Extra output filename(s) ignored: %s' % str(self.filenames[1:]),common.IncorrectValue,'provider')
+        log.error('Extra output filename(s) ignored: %s' % str(self.filenames[1:]),log.IncorrectValue,'provider')
 
     if self.format:
-      common.error('Specified output format(s) %s ignored' % self.format,common.IncorrectValue,'provider')
+      log.error('Specified output format(s) %s ignored' % self.format,log.IncorrectValue,'provider')
 
     # Creates a "ghost clean" topology after transformation
     # (AKA, remove unmanaged devices)
@@ -38,6 +40,7 @@ class ProviderConfiguration(_TopologyOutput):
     write_provider_file(providers.select_topology(topology,topology.provider),topology.provider,filename)
 
     for subprovider in topology[topology.provider].providers.keys():  # Iterate over subproviders
+      strings.print_colored_text('[INFO]    ','bright_cyan',alt_txt=None)
       print(f"Creating configuration file for secondary provider {subprovider}")
       write_provider_file(
         topology=providers.select_topology(topology,subprovider),
